@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dbridge.c	3.3	97/05/25	*/
+/*	SCCS Id: @(#)dbridge.c	3.4	2003/02/08	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -37,33 +37,42 @@ boolean
 is_pool(x,y)
 int x,y;
 {
-       register schar ltyp = levl[x][y].typ;
-       if(ltyp == POOL || ltyp == MOAT || ltyp == WATER) return TRUE;
-       if(ltyp == DRAWBRIDGE_UP &&
-	       (levl[x][y].drawbridgemask & DB_UNDER) == DB_MOAT) return TRUE;
-       return FALSE;
+    schar ltyp;
+
+    if (!isok(x,y)) return FALSE;
+    ltyp = levl[x][y].typ;
+    if (ltyp == POOL || ltyp == MOAT || ltyp == WATER) return TRUE;
+    if (ltyp == DRAWBRIDGE_UP &&
+	(levl[x][y].drawbridgemask & DB_UNDER) == DB_MOAT) return TRUE;
+    return FALSE;
 }
 
 boolean
 is_lava(x,y)
 int x,y;
 {
-       register schar ltyp = levl[x][y].typ;
-       if(ltyp == LAVAPOOL ||
-	  (ltyp == DRAWBRIDGE_UP &&
-	   (levl[x][y].drawbridgemask & DB_UNDER) == DB_LAVA)) return TRUE;
-       return FALSE;
+    schar ltyp;
+
+    if (!isok(x,y)) return FALSE;
+    ltyp = levl[x][y].typ;
+    if (ltyp == LAVAPOOL
+	|| (ltyp == DRAWBRIDGE_UP
+	    && (levl[x][y].drawbridgemask & DB_UNDER) == DB_LAVA)) return TRUE;
+    return FALSE;
 }
 
 boolean
 is_ice(x,y)
 int x,y;
 {
-	register schar ltyp = levl[x][y].typ;
-	if (ltyp == ICE ||
-	    (ltyp == DRAWBRIDGE_UP &&
-		(levl[x][y].drawbridgemask & DB_UNDER) == DB_ICE)) return TRUE;
-	return FALSE;
+    schar ltyp;
+
+    if (!isok(x,y)) return FALSE;
+    ltyp = levl[x][y].typ;
+    if (ltyp == ICE
+	|| (ltyp == DRAWBRIDGE_UP
+	    && (levl[x][y].drawbridgemask & DB_UNDER) == DB_ICE)) return TRUE;
+    return FALSE;
 }
 
 #endif /* OVL0 */
@@ -327,39 +336,14 @@ struct entity *etmp;
 const char *verb;
 {
 	static char wholebuf[80];
-	char verbbuf[30];
 
 	Strcpy(wholebuf, is_u(etmp) ? "You" : Monnam(etmp->emon));
-	if (!*verb)
-		return(wholebuf);
+	if (!*verb) return(wholebuf);
 	Strcat(wholebuf, " ");
-	verbbuf[0] = '\0';
 	if (is_u(etmp))
-		Strcpy(verbbuf, verb);
-	else {
-		if (!strcmp(verb, "are"))
-			Strcpy(verbbuf, "is");
-		if (!strcmp(verb, "have"))
-			Strcpy(verbbuf, "has");
-		if (!verbbuf[0]) {
-			Strcpy(verbbuf, verb);
-			switch (verbbuf[strlen(verbbuf) - 1]) {
-				case 'y':
-					verbbuf[strlen(verbbuf) - 1] = '\0';
-					Strcat(verbbuf, "ies");
-					break;
-				case 'h':
-				case 'o':
-				case 's':
-					Strcat(verbbuf, "es");
-					break;
-				default:
-					Strcat(verbbuf, "s");
-					break;
-			}
-		}
-	}
-	Strcat(wholebuf, verbbuf);
+	    Strcat(wholebuf, verb);
+	else
+	    Strcat(wholebuf, vtense((char *)0, verb));
 	return(wholebuf);
 }
 
@@ -396,22 +380,27 @@ struct entity *etmp;
 int dest, how;
 {
 	if (is_u(etmp)) {
-		if (how == DROWNING)
+		if (how == DROWNING) {
+			killer = 0;	/* drown() sets its own killer */
 			(void) drown();
-		else if (how == BURNING)
+		} else if (how == BURNING) {
+			killer = 0;	/* lava_effects() sets its own killer */
 			(void) lava_effects();
-		else {
+		} else {
 			coord xy;
 
-			killer_format = KILLED_BY_AN;
-			killer = "falling drawbridge";
+			/* use more specific killer if specified */
+			if (!killer) {
+			    killer_format = KILLED_BY_AN;
+			    killer = "falling drawbridge";
+			}
 			done(how);
 			/* So, you didn't die */
 			if (!e_survives_at(etmp, etmp->ex, etmp->ey)) {
 			    if (enexto(&xy, etmp->ex, etmp->ey, etmp->edata)) {
 				pline("A %s force teleports you away...",
 				      Hallucination ? "normal" : "strange");
-				teleds(xy.x, xy.y);
+				teleds(xy.x, xy.y, FALSE);
 			    }
 			    /* otherwise on top of the drawbridge is the
 			     * only viable spot in the dungeon, so stay there
@@ -421,6 +410,9 @@ int dest, how;
 		/* we might have crawled out of the moat to survive */
 		etmp->ex = u.ux,  etmp->ey = u.uy;
 	} else {
+		int entitycnt;
+
+		killer = 0;
 		/* fake "digested to death" damage-type suppresses corpse */
 #define mk_message(dest) ((dest & 1) ? "" : (char *)0)
 #define mk_corpse(dest)  ((dest & 2) ? AD_DGST : AD_PHYS)
@@ -430,6 +422,13 @@ int dest, how;
 		else		/* you caused it */
 		    xkilled(etmp->emon, dest);
 		etmp->edata = (struct permonst *)0;
+
+		/* dead long worm handling */
+		for (entitycnt = 0; entitycnt < ENTITIES; entitycnt++) {
+		    if (etmp != &(occupants[entitycnt]) &&
+			etmp->emon == occupants[entitycnt].emon)
+			occupants[entitycnt].edata = (struct permonst *)0;
+		}
 #undef mk_message
 #undef mk_corpse
 	}
@@ -541,6 +540,7 @@ struct entity *etmp;
 			pline_The("%s passes through %s!",
 			      at_portcullis ? "portcullis" : "drawbridge",
 			      e_nam(etmp));
+		if (is_u(etmp)) spoteffects(FALSE);
 		return;
 	}
 	if (e_missed(etmp, FALSE)) {
@@ -915,7 +915,7 @@ int x,y;
 		} else {
 			if (e_inview) {
 			    if (!is_u(etmp1) && Hallucination)
-				pline("%s into some heavy metal",
+				pline("%s into some heavy metal!",
 				      E_phrase(etmp1, "get"));
 			    else
 				pline("%s hit by a huge chunk of metal!",
