@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dungeon.c	3.3	1999/10/30	*/
+/*	SCCS Id: @(#)dungeon.c	3.4	1999/10/30	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -29,6 +29,14 @@ int n_dgns;				/* number of dungeons (used here,  */
 					/*   and mklev.c)		   */
 static branch *branches = (branch *) 0;	/* dungeon branch list		   */
 
+struct lchoice {
+	int idx;
+	schar lev[MAXLINFO];
+	schar playerlev[MAXLINFO];
+	xchar dgn[MAXLINFO];
+	char menuletter;
+};
+
 static void FDECL(Fread, (genericptr_t, int, int, dlb *));
 STATIC_DCL xchar FDECL(dname_to_dnum, (const char *));
 STATIC_DCL int FDECL(find_branch, (const char *, struct proto_dungeon *));
@@ -44,7 +52,7 @@ STATIC_DCL xchar FDECL(pick_level, (boolean *, int));
 STATIC_DCL boolean FDECL(place_level, (int, struct proto_dungeon *));
 #ifdef WIZARD
 STATIC_DCL const char *FDECL(br_string, (int));
-STATIC_DCL void FDECL(print_branch, (winid, int, int, int));
+STATIC_DCL void FDECL(print_branch, (winid, int, int, int, BOOLEAN_P, struct lchoice *));
 #endif
 
 #ifdef DEBUG
@@ -380,7 +388,10 @@ insert_branch(new_branch, extract_first)
     new_branch->next = (branch *) 0;
 
 /* Convert the branch into a unique number so we can sort them. */
-#define branch_val(bp) ((((long)(bp)->end1.dnum * (MAXLEVEL+1) + (long)(bp)->end1.dlevel) * (MAXDUNGEON+1) * (MAXLEVEL+1)) + ((long)(bp)->end2.dnum * (MAXLEVEL+1) + (long)(bp)->end2.dlevel))
+#define branch_val(bp) \
+	((((long)(bp)->end1.dnum * (MAXLEVEL+1) + \
+	  (long)(bp)->end1.dlevel) * (MAXDUNGEON+1) * (MAXLEVEL+1)) + \
+	 ((long)(bp)->end2.dnum * (MAXLEVEL+1) + (long)(bp)->end2.dlevel))
 
     /*
      * Insert the new branch into the correct place in the branch list.
@@ -483,6 +494,9 @@ init_level(dgn, proto_index, pd)
 	new_level->flags.maze_like = !!(tlevel->flags & MAZELIKE);
 	new_level->flags.rogue_like = !!(tlevel->flags & ROGUELIKE);
 	new_level->flags.align = ((tlevel->flags & D_ALIGN_MASK) >> 4);
+	if (!new_level->flags.align) 
+	    new_level->flags.align =
+		((pd->tmpdungeon[dgn].flags & D_ALIGN_MASK) >> 4);
 
 	new_level->rndlevs = tlevel->rndlevs;
 	new_level->next    = (s_level *) 0;
@@ -637,11 +651,39 @@ init_dungeons()		/* initialize the "dungeon" structs */
 	pd.n_levs = pd.n_brs = 0;
 
 	dgn_file = dlb_fopen(DUNGEON_FILE, RDBMODE);
-	if (!dgn_file)
-	    panic("Cannot open dungeon description file \"%s\"!", DUNGEON_FILE);
+	if (!dgn_file) {
+	    char tbuf[BUFSZ];
+	    Sprintf(tbuf, "Cannot open dungeon description - \"%s",
+		DUNGEON_FILE);
+#ifdef DLBRSRC /* using a resource from the executable */
+	    Strcat(tbuf, "\" resource!");
+#else /* using a file or DLB file */
+# if defined(DLB)
+	    Strcat(tbuf, "\" from ");
+#  ifdef PREFIXES_IN_USE
+	    Strcat(tbuf, "\n\"");
+	    if (fqn_prefix[DATAPREFIX]) Strcat(tbuf, fqn_prefix[DATAPREFIX]);
+#  else
+	    Strcat(tbuf, "\"");
+#  endif
+	    Strcat(tbuf, DLBFILE);
+# endif
+	    Strcat(tbuf, "\" file!");
+#endif
+#ifdef WIN32
+	    interject_assistance(1, INTERJECT_PANIC, (genericptr_t)tbuf,
+				 (genericptr_t)fqn_prefix[DATAPREFIX]);
+#endif
+	    panic(tbuf);
+	}
 
 	/* validate the data's version against the program's version */
 	Fread((genericptr_t) &vers_info, sizeof vers_info, 1, dgn_file);
+	/* we'd better clear the screen now, since when error messages come from
+	 * check_version() they will be printed using pline(), which doesn't
+	 * mix with the raw messages that might be already on the screen
+	 */
+	if (iflags.window_inited) clear_nhwindow(WIN_MAP);
 	if (!check_version(&vers_info, DUNGEON_FILE, TRUE))
 	    panic("Dungeon description not valid.");
 
@@ -715,7 +757,7 @@ init_dungeons()		/* initialize the "dungeon" structs */
 	    } else if (pd.tmpdungeon[i].entry_lev > 0) {
 		dungeons[i].entry_lev = pd.tmpdungeon[i].entry_lev;
 		if (dungeons[i].entry_lev > dungeons[i].num_dunlevs)
-		    dungeons[i].entry_lev = pd.tmpdungeon[i].entry_lev;
+		    dungeons[i].entry_lev = dungeons[i].num_dunlevs;
 	    } else { /* default */
 		dungeons[i].entry_lev = 1;	/* defaults to top level */
 	    }
@@ -1489,7 +1531,6 @@ const char *nam;
     return lev;
 }
 
-
 #ifdef WIZARD
 
 /* Convert a branch type to a string usable by print_dungeon(). */
@@ -1508,14 +1549,17 @@ br_string(type)
 
 /* Print all child branches between the lower and upper bounds. */
 STATIC_OVL void
-print_branch(win, dnum, lower_bound, upper_bound)
+print_branch(win, dnum, lower_bound, upper_bound, bymenu, lchoices)
     winid win;
     int   dnum;
     int   lower_bound;
     int   upper_bound;
+    boolean bymenu;
+    struct lchoice *lchoices;
 {
     branch *br;
     char buf[BUFSZ];
+    anything any;
 
     /* This assumes that end1 is the "parent". */
     for (br = branches; br; br = br->next) {
@@ -1525,14 +1569,29 @@ print_branch(win, dnum, lower_bound, upper_bound)
 		    br_string(br->type),
 		    dungeons[br->end2.dnum].dname,
 		    depth(&br->end1));
-	    putstr(win, 0, buf);
+	    if (bymenu) {
+		lchoices->lev[lchoices->idx] = br->end1.dlevel;
+		lchoices->dgn[lchoices->idx] = br->end1.dnum;
+		lchoices->playerlev[lchoices->idx] = depth(&br->end1);
+		any.a_void = 0;
+		any.a_int = lchoices->idx + 1;
+		add_menu(win, NO_GLYPH, &any, lchoices->menuletter,
+				0, ATR_NONE, buf, MENU_UNSELECTED);
+		if (lchoices->menuletter == 'z') lchoices->menuletter = 'A';
+		else lchoices->menuletter++;
+		lchoices->idx++;
+	    } else
+		putstr(win, 0, buf);
 	}
     }
 }
 
 /* Print available dungeon information. */
-void
-print_dungeon()
+schar
+print_dungeon(bymenu, rlev, rdgn)
+boolean bymenu;
+schar *rlev;
+xchar *rdgn;
 {
     int     i, last_level, nlev;
     char    buf[BUFSZ];
@@ -1540,7 +1599,15 @@ print_dungeon()
     s_level *slev;
     dungeon *dptr;
     branch  *br;
+    anything any;
+    struct lchoice lchoices;
+
     winid   win = create_nhwindow(NHW_MENU);
+    if (bymenu) {
+	start_menu(win);
+	lchoices.idx = 0;
+	lchoices.menuletter = 'a';
+    }
 
     for (i = 0, dptr = dungeons; i < n_dgns; i++, dptr++) {
 	nlev = dptr->num_dunlevs;
@@ -1558,7 +1625,11 @@ print_dungeon()
 		Sprintf(eos(buf), ", entrance on %d",
 			dptr->depth_start + dptr->entry_lev - 1);
 	}
-	putstr(win, 0, buf);
+	if (bymenu) {
+	    any.a_void = 0;
+	    add_menu(win, NO_GLYPH, &any, 0, 0, iflags.menu_headings, buf, MENU_UNSELECTED);
+	} else
+	    putstr(win, 0, buf);
 
 	/*
 	 * Circle through the special levels to find levels that are in
@@ -1568,32 +1639,72 @@ print_dungeon()
 	    if (slev->dlevel.dnum != i) continue;
 
 	    /* print any branches before this level */
-	    print_branch(win, i, last_level, slev->dlevel.dlevel);
+	    print_branch(win, i, last_level, slev->dlevel.dlevel, bymenu, &lchoices);
 
 	    Sprintf(buf, "   %s: %d", slev->proto, depth(&slev->dlevel));
 	    if (Is_stronghold(&slev->dlevel))
 		Sprintf(eos(buf), " (tune %s)", tune);
-	    putstr(win, 0, buf);
+	    if (bymenu) {
+	    	/* If other floating branches are added, this will need to change */
+	    	if (i != knox_level.dnum) {
+			lchoices.lev[lchoices.idx] = slev->dlevel.dlevel;
+			lchoices.dgn[lchoices.idx] = i;
+		} else {
+			lchoices.lev[lchoices.idx] = depth(&slev->dlevel);
+			lchoices.dgn[lchoices.idx] = 0;
+		}
+		lchoices.playerlev[lchoices.idx] = depth(&slev->dlevel);
+		any.a_void = 0;
+		any.a_int = lchoices.idx + 1;
+		add_menu(win, NO_GLYPH, &any, lchoices.menuletter,
+				0, ATR_NONE, buf, MENU_UNSELECTED);
+		if (lchoices.menuletter == 'z') lchoices.menuletter = 'A';
+		else lchoices.menuletter++;
+		lchoices.idx++;
+	    } else
+		putstr(win, 0, buf);
 
 	    last_level = slev->dlevel.dlevel;
 	}
 	/* print branches after the last special level */
-	print_branch(win, i, last_level, MAXLEVEL);
+	print_branch(win, i, last_level, MAXLEVEL, bymenu, &lchoices);
     }
 
     /* Print out floating branches (if any). */
     for (first = TRUE, br = branches; br; br = br->next) {
 	if (br->end1.dnum == n_dgns) {
 	    if (first) {
-		putstr(win, 0, "");
-		putstr(win, 0, "Floating branches");
+	    	if (!bymenu) {
+		    putstr(win, 0, "");
+		    putstr(win, 0, "Floating branches");
+		}
 		first = FALSE;
 	    }
 	    Sprintf(buf, "   %s to %s",
 			br_string(br->type), dungeons[br->end2.dnum].dname);
-	    putstr(win, 0, buf);
+	    if (!bymenu)
+		putstr(win, 0, buf);
 	}
     }
+    if (bymenu) {
+    	int n;
+	menu_item *selected;
+	int idx;
+
+	end_menu(win, "Level teleport to where:");
+	n = select_menu(win, PICK_ONE, &selected);
+	destroy_nhwindow(win);
+	if (n > 0) {
+		idx = selected[0].item.a_int - 1;
+		free((genericptr_t)selected);
+		if (rlev && rdgn) {
+			*rlev = lchoices.lev[idx];
+			*rdgn = lchoices.dgn[idx];
+			return lchoices.playerlev[idx];
+		}
+	}
+	return 0;
+    }
 
     /* I hate searching for the invocation pos while debugging. -dean */
     if (Invocation_lev(&u.uz)) {
@@ -1624,6 +1735,7 @@ print_dungeon()
 
     display_nhwindow(win, TRUE);
     destroy_nhwindow(win);
+    return 0;
 }
 #endif /* WIZARD */
 
