@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)hack.c	3.4	2002/03/09	*/
+/*	SCCS Id: @(#)hack.c	3.4	2003/04/30	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -12,7 +12,7 @@ STATIC_DCL int FDECL(still_chewing,(XCHAR_P,XCHAR_P));
 #ifdef SINKS
 STATIC_DCL void NDECL(dosinkfall);
 #endif
-STATIC_DCL void NDECL(findtravelpath);
+STATIC_DCL boolean FDECL(findtravelpath, (BOOLEAN_P));
 STATIC_DCL boolean FDECL(monstinroom, (struct permonst *,int));
 
 STATIC_DCL void FDECL(move_update, (BOOLEAN_P));
@@ -47,7 +47,7 @@ const char *msg;
     /* this location might not be safe, if not, move revived monster */
     if (revived) {
 	mtmp = m_at(x,y);
-	if (mtmp && !goodpos(x, y, mtmp) &&
+	if (mtmp && !goodpos(x, y, mtmp, 0) &&
 	    enexto(&cc, x, y, mtmp->data)) {
 	    rloc_to(mtmp, cc.x, cc.y);
 	}
@@ -89,6 +89,7 @@ moverock()
 	    goto cannot_push;
 	}
 	if (isok(rx,ry) && !IS_ROCK(levl[rx][ry].typ) &&
+	    levl[rx][ry].typ != IRONBARS &&
 	    (!IS_DOOR(levl[rx][ry].typ) || !(u.dx && u.dy) || (
 #ifdef REINCARNATION
 		!Is_rogue_level(&u.uz) &&
@@ -113,17 +114,17 @@ moverock()
 		    (!mtmp->mtrapped ||
 			 !(ttmp && ((ttmp->ttyp == PIT) ||
 				    (ttmp->ttyp == SPIKED_PIT))))) {
+		if (Blind) feel_location(sx, sy);
 		if (canspotmon(mtmp))
-		    pline("There's %s on the other side.", mon_nam(mtmp));
+		    pline("There's %s on the other side.", a_monnam(mtmp));
 		else {
-		    if (Blind) feel_location(sx, sy);
 		    You_hear("a monster behind %s.", the(xname(otmp)));
 		    map_invisible(rx, ry);
 		}
 		if (flags.verbose)
 		    pline("Perhaps that's why %s cannot move it.",
 #ifdef STEED
-				u.usteed ? mon_nam(u.usteed) :
+				u.usteed ? y_monnam(u.usteed) :
 #endif
 				"you");
 		goto cannot_push;
@@ -133,11 +134,13 @@ moverock()
 		switch(ttmp->ttyp) {
 		case LANDMINE:
 		    if (rn2(10)) {
-			pline("KAABLAMM!!!  %s %s land mine.",
-				Tobjnam(otmp, "trigger"),
-				ttmp->madeby_u ? "your" : "a");
 			obj_extract_self(otmp);
 			place_object(otmp, rx, ry);
+			unblock_point(sx, sy);
+			newsym(sx, sy);
+			pline("KAABLAMM!!!  %s %s land mine.",
+			      Tobjnam(otmp, "trigger"),
+			      ttmp->madeby_u ? "your" : "a");
 			blow_up_landmine(ttmp);
 			/* if the boulder remains, it should fill the pit */
 			fill_pit(u.ux, u.uy);
@@ -180,7 +183,7 @@ moverock()
 #ifdef STEED
 		    if (u.usteed)
 			pline("%s pushes %s and suddenly it disappears!",
-				Monnam(u.usteed), the(xname(otmp)));
+			      upstart(y_monnam(u.usteed)), the(xname(otmp)));
 		    else
 #endif
 		    You("push %s and suddenly it disappears!",
@@ -221,10 +224,9 @@ moverock()
 		long lastmovetime;
 		lastmovetime = 0;
 #else
+		/* note: reset to zero after save/restore cycle */
 		static NEARDATA long lastmovetime;
 #endif
-		/* note: this var contains garbage initially and
-		   after a restore */
 #ifdef STEED
 		if (!u.usteed) {
 #endif
@@ -235,7 +237,8 @@ moverock()
 		  exercise(A_STR, TRUE);
 #ifdef STEED
 		} else 
-		    pline("%s moves %s.", Monnam(u.usteed), the(xname(otmp)));
+		    pline("%s moves %s.",
+			  upstart(y_monnam(u.usteed)), the(xname(otmp)));
 #endif
 		lastmovetime = moves;
 	    }
@@ -255,7 +258,7 @@ moverock()
 #ifdef STEED
 	  if (u.usteed)
 	    pline("%s tries to move %s, but cannot.",
-		  	Monnam(u.usteed), the(xname(otmp)));
+		  upstart(y_monnam(u.usteed)), the(xname(otmp)));
 	  else
 #endif
 	    You("try to move %s, but in vain.", the(xname(otmp)));
@@ -267,7 +270,7 @@ moverock()
 		    You("aren't skilled enough to %s %s from %s.",
 			(flags.pickup && !In_sokoban(&u.uz))
 			    ? "pick up" : "push aside",
-			the(xname(otmp)), mon_nam(u.usteed));
+			the(xname(otmp)), y_monnam(u.usteed));
 		} else
 #endif
 		{
@@ -381,12 +384,15 @@ still_chewing(x,y)
 	digtxt = "chew a hole in the wall.";
 	if (level.flags.is_maze_lev) {
 	    lev->typ = ROOM;
-	} else if (level.flags.is_cavernous_lev) {
+	} else if (level.flags.is_cavernous_lev && !in_town(x, y)) {
 	    lev->typ = CORR;
 	} else {
 	    lev->typ = DOOR;
 	    lev->doormask = D_NODOOR;
 	}
+    } else if (IS_TREE(lev->typ)) {
+	digtxt = "chew through the tree.";
+	lev->typ = ROOM;
     } else if (lev->typ == SDOOR) {
 	if (lev->doormask & D_TRAPPED) {
 	    lev->doormask = D_NODOOR;
@@ -418,7 +424,7 @@ still_chewing(x,y)
     unblock_point(x, y);	/* vision */
     newsym(x, y);
     if (digtxt) You(digtxt);	/* after newsym */
-    if (dmgtxt) pay_for_damage(dmgtxt);
+    if (dmgtxt) pay_for_damage(dmgtxt, FALSE);
     (void) memset((genericptr_t)&digging, 0, sizeof digging);
     return 0;
 }
@@ -449,6 +455,13 @@ dosinkfall()
 	if (is_floater(youmonst.data) || (HLevitation & FROMOUTSIDE)) {
 	    You("wobble unsteadily for a moment.");
 	} else {
+	    long save_ELev = ELevitation, save_HLev = HLevitation;
+
+	    /* fake removal of levitation in advance so that final
+	       disclosure will be right in case this turns out to
+	       be fatal; fortunately the fact that rings and boots
+	       are really still worn has no effect on bones data */
+	    ELevitation = HLevitation = 0L;
 	    You("crash to the floor!");
 	    losehp(rn1(8, 25 - (int)ACURR(A_CON)),
 		   fell_on_sink, NO_KILLER_PREFIX);
@@ -460,6 +473,8 @@ dosinkfall()
 		    losehp(rnd(3), fell_on_sink, NO_KILLER_PREFIX);
 		    exercise(A_CON, FALSE);
 		}
+	    ELevitation = save_ELev;
+	    HLevitation = save_HLev;
 	}
 
 	ELevitation &= ~W_ARTI;
@@ -525,11 +540,13 @@ xchar x, y;
 #endif /* OVL1 */
 #ifdef OVL3
 
-/* return TRUE if (dx,dy) is an OK place to move */
+/* return TRUE if (dx,dy) is an OK place to move
+ * mode is one of DO_MOVE, TEST_MOVE or TEST_TRAV
+ */
 boolean 
-test_move(ux, uy, dx, dy, test_only)
+test_move(ux, uy, dx, dy, mode)
 int ux, uy, dx, dy;
-boolean test_only;
+int mode;
 {
     int x = ux+dx;
     int y = uy+dy;
@@ -540,93 +557,107 @@ boolean test_only;
      *  Check for physical obstacles.  First, the place we are going.
      */
     if (IS_ROCK(tmpr->typ) || tmpr->typ == IRONBARS) {
-	if (Blind && !test_only) feel_location(x,y);
+	if (Blind && mode == DO_MOVE) feel_location(x,y);
 	if (Passes_walls && may_passwall(x,y)) {
 	    ;	/* do nothing */
+	} else if (tmpr->typ == IRONBARS) {
+	    if (!(Passes_walls || passes_bars(youmonst.data)))
+		return FALSE;
 	} else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
 	    /* Eat the rock. */
-	    if (!test_only && still_chewing(x,y)) return FALSE;
+	    if (mode == DO_MOVE && still_chewing(x,y)) return FALSE;
 	} else if (flags.autodig && !flags.run && !flags.nopick &&
 		   uwep && is_pick(uwep)) {
 	/* MRKR: Automatic digging when wielding the appropriate tool */
-	    if (!test_only)
+	    if (mode == DO_MOVE)
 		(void) use_pick_axe2(uwep);
 	    return FALSE;
 	} else {
-	    if ( !test_only ) {
+	    if (mode == DO_MOVE) {
 		if (Is_stronghold(&u.uz) && is_db_wall(x,y))
 		    pline_The("drawbridge is up!");
 		if (Passes_walls && !may_passwall(x,y) && In_sokoban(&u.uz))
-	    		pline_The("Sokoban walls resist your ability.");
+		    pline_The("Sokoban walls resist your ability.");
 	    }
 	    return FALSE;
 	}
     } else if (IS_DOOR(tmpr->typ)) {
 	if (closed_door(x,y)) {
-	    if (Blind && !test_only) feel_location(x,y);
+	    if (Blind && mode == DO_MOVE) feel_location(x,y);
 	    if (Passes_walls)
 		;	/* do nothing */
 	    else if (can_ooze(&youmonst)) {
-		if ( !test_only ) You("ooze under the door.");
+		if (mode == DO_MOVE) You("ooze under the door.");
 	    } else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
 		/* Eat the door. */
-		if (!test_only && still_chewing(x,y)) return FALSE;
+		if (mode == DO_MOVE && still_chewing(x,y)) return FALSE;
 	    } else {
-		if ( !test_only ) {
+		if (mode == DO_MOVE) {
 		    if (amorphous(youmonst.data))
 			You("try to ooze under the door, but can't squeeze your possessions through.");
 		    else if (x == ux || y == uy) {
 			if (Blind || Stunned || ACURR(A_DEX) < 10 || Fumbling) {
 #ifdef STEED
-			    if (u.usteed)
+			    if (u.usteed) {
 				You_cant("lead %s through that closed door.",
-				      x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE));
-		 	    else {
-#else
+				         y_monnam(u.usteed));
+			    } else
+#endif
+			    {
 			        pline("Ouch!  You bump into a door.");
 			        exercise(A_DEX, FALSE);
-#endif
-#ifdef STEED
 			    }
-#endif
 			} else pline("That door is closed.");
 		    }
-		}
+		} else if (mode == TEST_TRAV) goto testdiag;
 		return FALSE;
 	    }
-	} else if (dx && dy && !Passes_walls
-		    && ((tmpr->doormask & ~D_BROKEN)
+	} else {
+	testdiag:
+	    if (dx && dy && !Passes_walls
+		&& ((tmpr->doormask & ~D_BROKEN)
 #ifdef REINCARNATION
-				    || Is_rogue_level(&u.uz)
+		    || Is_rogue_level(&u.uz)
 #endif
-				    || block_door(x,y))) {
-	    /* Diagonal moves into a door are not allowed. */
-	    if ( Blind && !test_only )
-		feel_location(x,y);
-	    return FALSE;
+		    || block_door(x,y))) {
+		/* Diagonal moves into a door are not allowed. */
+		if (Blind && mode == DO_MOVE)
+		    feel_location(x,y);
+		return FALSE;
+	    }
 	}
     }
     if (dx && dy
 	    && bad_rock(youmonst.data,ux,y) && bad_rock(youmonst.data,x,uy)) {
 	/* Move at a diagonal. */
 	if (In_sokoban(&u.uz)) {
-	    if ( !test_only )
+	    if (mode == DO_MOVE)
 		You("cannot pass that way.");
 	    return FALSE;
 	}
 	if (bigmonst(youmonst.data)) {
-	    if ( !test_only )
+	    if (mode == DO_MOVE)
 		Your("body is too large to fit through.");
 	    return FALSE;
 	}
 	if (invent && (inv_weight() + weight_cap() > 600)) {
-	    if ( !test_only )
+	    if (mode == DO_MOVE)
 		You("are carrying too much to get through.");
 	    return FALSE;
 	}
     }
+    /* Pick travel path that does not require crossing a trap.
+     * Avoid water and lava using the usual running rules.
+     * (but not u.ux/u.uy because findtravelpath walks toward u.ux/u.uy) */
+    if (flags.run == 8 && mode != DO_MOVE && (x != u.ux || y != u.uy)) {
+	struct trap* t = t_at(x, y);
+
+	if ((t && t->tseen) ||
+	    (!Levitation && !Flying &&
+	     !is_clinger(youmonst.data) &&
+	     (is_pool(x, y) || is_lava(x, y)) && levl[x][y].seenv))
+	    return FALSE;
+    }
 
     ust = &levl[ux][uy];
 
@@ -643,80 +674,189 @@ boolean test_only;
     }
 
     if (sobj_at(BOULDER,x,y) && (In_sokoban(&u.uz) || !Passes_walls)) {
-	if (!(Blind || Hallucination) && (flags.run >= 2))
+	if (!(Blind || Hallucination) && (flags.run >= 2) && mode != TEST_TRAV)
 	    return FALSE;
-	if (!test_only) {
+	if (mode == DO_MOVE) {
 	    /* tunneling monsters will chew before pushing */
 	    if (tunnels(youmonst.data) && !needspick(youmonst.data) &&
 		!In_sokoban(&u.uz)) {
 		if (still_chewing(x,y)) return FALSE;
 	    } else
 		if (moverock() < 0) return FALSE;
+	} else if (mode == TEST_TRAV) {
+	    struct obj* obj;
+
+	    /* don't pick two boulders in a row, unless there's a way thru */
+	    if (sobj_at(BOULDER,ux,uy) && !In_sokoban(&u.uz)) {
+		if (!Passes_walls &&
+		    !(tunnels(youmonst.data) && !needspick(youmonst.data)) &&
+		    !carrying(PICK_AXE) && !carrying(DWARVISH_MATTOCK) &&
+		    !((obj = carrying(WAN_DIGGING)) &&
+		      !objects[obj->otyp].oc_name_known))
+		    return FALSE;
+	    }
 	}
-	/* test_only will assume you'll be able to push it when you get there... */
+	/* assume you'll be able to push it when you get there... */
     }
 
     /* OK, it is a legal place to move. */
     return TRUE;
 }
 
-static void findtravelpath()
+/*
+ * Find a path from the destination (u.tx,u.ty) back to (u.ux,u.uy).
+ * A shortest path is returned.  If guess is TRUE, consider various
+ * inaccessible locations as valid intermediate path points.
+ * Returns TRUE if a path was found.
+ */
+static boolean
+findtravelpath(guess)
+boolean guess;
 {
-    if ( u.tx != u.ux || u.ty != u.uy ) {
+    /* if travel to adjacent, reachable location, use normal movement rules */
+    if (!guess && iflags.travel1 && distmin(u.ux, u.uy, u.tx, u.ty) == 1) {
+	flags.run = 0;
+	if (test_move(u.ux, u.uy, u.tx-u.ux, u.ty-u.uy, TEST_MOVE)) {
+	    u.dx = u.tx-u.ux;
+	    u.dy = u.ty-u.uy;
+	    nomul(0);
+	    iflags.travelcc.x = iflags.travelcc.y = -1;
+	    return TRUE;
+	}
+	flags.run = 8;
+    }
+    if (u.tx != u.ux || u.ty != u.uy) {
 	xchar travel[COLNO][ROWNO];
 	xchar travelstepx[2][COLNO*ROWNO];
 	xchar travelstepy[2][COLNO*ROWNO];
-	int n=1;
-	int set=0;
-	int dia=1;
+	xchar tx, ty, ux, uy;
+	int n = 1;			/* max offset in travelsteps */
+	int set = 0;			/* two sets current and previous */
+	int radius = 1;			/* search radius */
+	int i;
 
-	(void) memset((genericptr_t)travel,0,sizeof(travel));
+	/* If guessing, first find an "obvious" goal location.  The obvious
+	 * goal is the position the player knows of, or might figure out
+	 * (couldsee) that is closest to the target on a straight path.
+	 */
+	if (guess) {
+	    tx = u.ux; ty = u.uy; ux = u.tx; uy = u.ty;
+	} else {
+	    tx = u.tx; ty = u.ty; ux = u.ux; uy = u.uy;
+	}
 
-	travelstepx[0][0] = u.tx;
-	travelstepy[0][0] = u.ty;
-	while ( n ) {
-	    int i;
-	    int nn=0;
-	    for (i=0; i<n; i++) {
+    noguess:
+	(void) memset((genericptr_t)travel, 0, sizeof(travel));
+	travelstepx[0][0] = tx;
+	travelstepy[0][0] = ty;
+
+	while (n != 0) {
+	    int nn = 0;
+
+	    for (i = 0; i < n; i++) {
 		int dir;
 		int x = travelstepx[set][i];
 		int y = travelstepy[set][i];
 		static int ordered[] = { 0, 2, 4, 6, 1, 3, 5, 7 };
-		for (dir=0; dir<8; dir++) {
+		/* no diagonal movement for grid bugs */
+		int dirmax = u.umonnum == PM_GRID_BUG ? 4 : 8;
+
+		for (dir = 0; dir < dirmax; ++dir) {
 		    int nx = x+xdir[ordered[dir]];
 		    int ny = y+ydir[ordered[dir]];
-    /*printf("try %d,%d\n",nx,ny);*/
-		    if ( isok(nx,ny) && test_move( x, y, nx-x, ny-y, 1 ) ) {
-			if ( nx == u.ux && ny == u.uy ) {
-			    u.dx = x-u.ux;
-			    u.dy = y-u.uy;
-	/*printf("found\n");*/
-	/*printf("findtravelpath %d,%d -> %d,%d by %d,%d\n",u.ux,u.uy,u.tx,u.ty,u.dx,u.dy);
-	*/
-			    return;
-			} else {
-	/*printf("%d %d %d",isok(nx,ny), !travel[nx][ny], ACCESSIBLE(levl[nx][ny].typ));*/
-			    if ( !travel[nx][ny] ) {
-				travelstepx[1-set][nn]=nx;
-				travelstepy[1-set][nn]=ny;
-				travel[nx][ny]=dia;
-				nn++;
+
+		    if (!isok(nx, ny)) continue;
+		    if ((!Passes_walls && !can_ooze(&youmonst) &&
+			closed_door(x, y)) || sobj_at(BOULDER, x, y)) {
+			/* closed doors and boulders usually
+			 * cause a delay, so prefer another path */
+			if (travel[x][y] > radius-3) {
+			    travelstepx[1-set][nn] = x;
+			    travelstepy[1-set][nn] = y;
+			    /* don't change travel matrix! */
+			    nn++;
+			    continue;
+			}
+		    }
+		    if (test_move(x, y, nx-x, ny-y, TEST_TRAV) &&
+			(levl[nx][ny].seenv || (!Blind && couldsee(nx, ny)))) {
+			if (nx == ux && ny == uy) {
+			    if (!guess) {
+				u.dx = x-ux;
+				u.dy = y-uy;
+				if (x == u.tx && y == u.ty) {
+				    nomul(0);
+				    /* reset run so domove run checks work */
+				    flags.run = 8;
+				    iflags.travelcc.x = iflags.travelcc.y = -1;
+				}
+				return TRUE;
 			    }
+			} else if (!travel[nx][ny]) {
+			    travelstepx[1-set][nn] = nx;
+			    travelstepy[1-set][nn] = ny;
+			    travel[nx][ny] = radius;
+			    nn++;
 			}
 		    }
 		}
 	    }
+	    
 	    n = nn;
 	    set = 1-set;
-	    dia++;
+	    radius++;
 	}
 
-	/* give up */
+	/* if guessing, find best location in travel matrix and go there */
+	if (guess) {
+	    int px = tx, py = ty;	/* pick location */
+	    int dist, nxtdist, d2, nd2;
+
+	    dist = distmin(ux, uy, tx, ty);
+	    d2 = dist2(ux, uy, tx, ty);
+	    for (tx = 1; tx < COLNO; ++tx)
+		for (ty = 0; ty < ROWNO; ++ty)
+		    if (travel[tx][ty]) {
+			nxtdist = distmin(ux, uy, tx, ty);
+			if (nxtdist == dist && couldsee(tx, ty)) {
+			    nd2 = dist2(ux, uy, tx, ty);
+			    if (nd2 < d2) {
+				/* prefer non-zigzag path */
+				px = tx; py = ty;
+				d2 = nd2;
+			    }
+			} else if (nxtdist < dist && couldsee(tx, ty)) {
+			    px = tx; py = ty;
+			    dist = nxtdist;
+			    d2 = dist2(ux, uy, tx, ty);
+			}
+		    }
+
+	    if (px == u.ux && py == u.uy) {
+		/* no guesses, just go in the general direction */
+		u.dx = sgn(u.tx - u.ux);
+		u.dy = sgn(u.ty - u.uy);
+		if (test_move(u.ux, u.uy, u.dx, u.dy, TEST_MOVE))
+		    return TRUE;
+		goto found;
+	    }
+	    tx = px;
+	    ty = py;
+	    ux = u.ux;
+	    uy = u.uy;
+	    set = 0;
+	    n = radius = 1;
+	    guess = FALSE;
+	    goto noguess;
+	}
+	return FALSE;
     }
 
+found:
     u.dx = 0;
     u.dy = 0;
     nomul(0);
+    return FALSE;
 }
 
 void
@@ -731,11 +871,15 @@ domove()
 	xchar chainx, chainy, ballx, bally;	/* ball&chain new positions */
 	int bc_control;				/* control for ball&chain */
 	boolean cause_delay = FALSE;	/* dragging ball will skip a move */
+	const char *predicament;
 
 	u_wipe_engr(rnd(5));
 
-	if ( flags.travel )
-	    findtravelpath();
+	if (flags.travel) {
+	    if (!findtravelpath(FALSE))
+		(void) findtravelpath(TRUE);
+	    iflags.travel1 = 0;
+	}
 
 	if(((wtcap = near_capacity()) >= OVERLOADED
 	    || (wtcap > SLT_ENCUMBER &&
@@ -821,7 +965,10 @@ domove()
 			nomul(0);
 			return;
 		}
-		if((trap = t_at(x, y)) && trap->tseen) {
+		if (((trap = t_at(x, y)) && trap->tseen) ||
+		    (Blind && !Levitation && !Flying &&
+		     !is_clinger(youmonst.data) &&
+		     (is_pool(x, y) || is_lava(x, y)) && levl[x][y].seenv)) {
 			if(flags.run >= 2) {
 				nomul(0);
 				flags.move = 0;
@@ -952,13 +1099,20 @@ domove()
 	if (flags.forcefight ||
 	    /* remembered an 'I' && didn't use a move command */
 	    (glyph_is_invisible(levl[x][y].glyph) && !flags.nopick)) {
+		boolean expl = (Upolyd && attacktype(youmonst.data, AT_EXPL));
 	    	char buf[BUFSZ];
 		Sprintf(buf,"a vacant spot on the %s", surface(x,y));
-		You("attack %s.",
-		    !Underwater ? "thin air" : is_pool(x,y) ? "empty water" : buf);
+		You("%s %s.",
+		    expl ? "explode at" : "attack",
+		    !Underwater ? "thin air" :
+		    is_pool(x,y) ? "empty water" : buf);
 		unmap_object(x, y); /* known empty -- remove 'I' if present */
 		newsym(x, y);
 		nomul(0);
+		if (expl) {
+		    u.mh = -1;		/* dead in the current form */
+		    rehumanize();
+		}
 		return;
 	}
 	if (glyph_is_invisible(levl[x][y].glyph)) {
@@ -968,10 +1122,7 @@ domove()
 	/* not attacking an animal, so we try to move */
 #ifdef STEED
 	if (u.usteed && !u.usteed->mcanmove && (u.dx || u.dy)) {
-		pline("%s won't move!",
-			upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+		pline("%s won't move!", upstart(y_monnam(u.usteed)));
 		nomul(0);
 		return;
 	} else
@@ -1003,10 +1154,8 @@ domove()
 		    } else if (flags.verbose) {
 #ifdef STEED
 			if (u.usteed)
-			    	Norep("%s is still in a pit.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			    Norep("%s is still in a pit.",
+				  upstart(y_monnam(u.usteed)));
 			else
 #endif
 			Norep( (Hallucination && !rn2(5)) ?
@@ -1015,14 +1164,11 @@ domove()
 		    }
 		} else if (u.utraptype == TT_LAVA) {
 		    if(flags.verbose) {
-			char *predicament = "stuck in the lava";
+			predicament = "stuck in the lava";
 #ifdef STEED
 			if (u.usteed)
-			    	Norep("%s is %s.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
-					 predicament);
+			    Norep("%s is %s.", upstart(y_monnam(u.usteed)),
+				  predicament);
 			else
 #endif
 			Norep("You are %s.", predicament);
@@ -1033,9 +1179,7 @@ domove()
 #ifdef STEED
 			    if (u.usteed)
 				You("lead %s to the edge of the lava.",
-				      x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE));
+				    y_monnam(u.usteed));
 			    else
 #endif
 			     You("pull yourself to the edge of the lava.");
@@ -1051,14 +1195,11 @@ domove()
 		    }
 		    if(--u.utrap) {
 			if(flags.verbose) {
-			    char *predicament = "stuck to the web";
+			    predicament = "stuck to the web";
 #ifdef STEED
 			    if (u.usteed)
-			    	Norep("%s is %s.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
-					 predicament);
+				Norep("%s is %s.", upstart(y_monnam(u.usteed)),
+				      predicament);
 			    else
 #endif
 			    Norep("You are %s.", predicament);
@@ -1066,10 +1207,8 @@ domove()
 		    } else {
 #ifdef STEED
 			if (u.usteed)
-				pline("%s breaks out of the web.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			    pline("%s breaks out of the web.",
+				  upstart(y_monnam(u.usteed)));
 			else
 #endif
 			You("disentangle yourself.");
@@ -1077,39 +1216,33 @@ domove()
 		} else if (u.utraptype == TT_INFLOOR) {
 		    if(--u.utrap) {
 			if(flags.verbose) {
-			    char *predicament = "stuck in the";
+			    predicament = "stuck in the";
 #ifdef STEED
 			    if (u.usteed)
-			    	Norep("%s is %s %s.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
-					 predicament, surface(u.ux, u.uy));
+				Norep("%s is %s %s.",
+				      upstart(y_monnam(u.usteed)),
+				      predicament, surface(u.ux, u.uy));
 			    else
 #endif
-			    Norep("You are %s %s.", predicament, surface(u.ux, u.uy));
+			    Norep("You are %s %s.", predicament,
+				  surface(u.ux, u.uy));
 			}
 		    } else {
 #ifdef STEED
 			if (u.usteed)
-				pline("%s finally wiggles free.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			    pline("%s finally wiggles free.",
+				  upstart(y_monnam(u.usteed)));
 			else
 #endif
 			You("finally wiggle free.");
 		    }
 		} else {
 		    if(flags.verbose) {
-			char *predicament = "caught in a bear trap";
+			predicament = "caught in a bear trap";
 #ifdef STEED
 			if (u.usteed)
-			    	Norep("%s is %s.",
-				      upstart(x_monnam(u.usteed,
-					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
-				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
-					 predicament);
+			    Norep("%s is %s.", upstart(y_monnam(u.usteed)),
+				  predicament);
 			else
 #endif
 			Norep("You are %s.", predicament);
@@ -1119,7 +1252,7 @@ domove()
 		return;
 	}
 
-	if ( !test_move( u.ux, u.uy, x-u.ux, y-u.uy, 0 ) ) {
+	if (!test_move(u.ux, u.uy, x-u.ux, y-u.uy, DO_MOVE)) {
 	    flags.move = 0;
 	    nomul(0);
 	    return;
@@ -1128,7 +1261,7 @@ domove()
 	/* Move ball and chain.  */
 	if (Punished)
 	    if (!drag_ball(x,y, &bc_control, &ballx, &bally, &chainx, &chainy,
-			&cause_delay))
+			&cause_delay, TRUE))
 		return;
 
 	/* Check regions entering/leaving */
@@ -1182,6 +1315,13 @@ domove()
 		    sobj_at(BOULDER, trap->tx, trap->ty)) {
 		/* can't swap places with pet pinned in a pit by a boulder */
 		u.ux = u.ux0,  u.uy = u.uy0;	/* didn't move after all */
+	    } else if (u.ux0 != x && u.uy0 != y &&
+		       bad_rock(mtmp->data, x, u.uy0) &&
+		       bad_rock(mtmp->data, u.ux0, y) &&
+		       (bigmonst(mtmp->data) || (curr_mon_load(mtmp) > 600))) {
+		/* can't swap places when pet won't fit thru the opening */
+		u.ux = u.ux0,  u.uy = u.uy0;	/* didn't move after all */
+		You("stop.  %s won't fit through.", upstart(y_monnam(mtmp)));
 	    } else {
 		char pnambuf[BUFSZ];
 
@@ -1271,6 +1411,21 @@ domove()
 	    nomul(-2);
 	    nomovemsg = "";
 	}
+
+	if (flags.run && iflags.runmode != RUN_TPORT) {
+	    /* display every step or every 7th step depending upon mode */
+	    if (iflags.runmode != RUN_LEAP || !(moves % 7L)) {
+		if (flags.time) flags.botl = 1;
+		curs_on_u();
+		delay_output();
+		if (iflags.runmode == RUN_CRAWL) {
+		    delay_output();
+		    delay_output();
+		    delay_output();
+		    delay_output();
+		}
+	    }
+	}
 }
 
 void
@@ -1278,10 +1433,18 @@ invocation_message()
 {
 	/* a special clue-msg when on the Invocation position */
 	if(invocation_pos(u.ux, u.uy) && !On_stairs(u.ux, u.uy)) {
+	    char buf[BUFSZ];
 	    struct obj *otmp = carrying(CANDELABRUM_OF_INVOCATION);
 
-	    You_feel("a strange vibration under your %s.",
-		    makeplural(body_part(FOOT)));
+	    nomul(0);		/* stop running or travelling */
+#ifdef STEED
+	    if (u.usteed) Sprintf(buf, "beneath %s", y_monnam(u.usteed));
+	    else
+#endif
+	    if (Levitation || Flying) Strcpy(buf, "beneath you");
+	    else Sprintf(buf, "under your %s", makeplural(body_part(FOOT)));
+
+	    You_feel("a strange vibration %s.", buf);
 	    if (otmp && otmp->spe == 7 && otmp->lamplit)
 		pline("%s %s!", The(xname(otmp)),
 		    Blind ? "throbs palpably" : "glows with a strange light");
@@ -1295,7 +1458,6 @@ void
 spoteffects(pick)
 boolean pick;
 {
-	register struct trap *trap;
 	register struct monst *mtmp;
 
 	if(u.uinwater) {
@@ -1353,11 +1515,16 @@ stillinwater:;
 	if(IS_SINK(levl[u.ux][u.uy].typ) && Levitation)
 		dosinkfall();
 #endif
-	if (pick && !in_steed_dismounting)
-		(void) pickup(1);
-	/* if dismounting, we'll check again later */
-	if ((trap = t_at(u.ux,u.uy)) != 0 && !in_steed_dismounting)
-		dotrap(trap, 0);	/* fall into pit, arrow trap, etc. */
+	if (!in_steed_dismounting) { /* if dismounting, we'll check again later */
+		struct trap *trap = t_at(u.ux, u.uy);
+		boolean pit;
+		pit = (trap && (trap->ttyp == PIT || trap->ttyp == SPIKED_PIT));
+		if (trap && pit)
+			dotrap(trap, 0);	/* fall into pit */
+		if (pick) (void) pickup(1);
+		if (trap && !pit)
+			dotrap(trap, 0);	/* fall into arrow trap, etc. */
+	}
 	if((mtmp = m_at(u.ux, u.uy)) && !u.uswallow) {
 		mtmp->mundetected = mtmp->msleeping = 0;
 		switch(mtmp->data->mlet) {
@@ -1366,7 +1533,7 @@ stillinwater:;
 			      Amonnam(mtmp), ceiling(u.ux,u.uy));
 			if(mtmp->mtame) /* jumps to greet you, not attack */
 			    ;
-			else if(uarmh)
+			else if(uarmh && is_metallic(uarmh))
 			    pline("Its blow glances off your helmet.");
 			else if (u.uac + 3 <= rnd(20))
 			    You("are almost hit by %s!",
@@ -1480,6 +1647,31 @@ register int typewanted;
 	return(ptr);
 }
 
+/* is (x,y) in a town? */
+boolean
+in_town(x, y)
+register int x, y;
+{
+	s_level *slev = Is_special(&u.uz);
+	register struct mkroom *sroom;
+	boolean has_subrooms = FALSE;
+
+	if (!slev || !slev->flags.town) return FALSE;
+
+	/*
+	 * See if (x,y) is in a room with subrooms, if so, assume it's the
+	 * town.  If there are no subrooms, the whole level is in town.
+	 */
+	for (sroom = &rooms[0]; sroom->hx > 0; sroom++) {
+	    if (sroom->nsubrooms > 0) {
+		has_subrooms = TRUE;
+		if (inside_room(sroom, x, y)) return TRUE;
+	    }
+	}
+
+	return !has_subrooms;
+}
+
 STATIC_OVL void
 move_update(newlev)
 register boolean newlev;
@@ -1644,7 +1836,8 @@ int
 dopickup()
 {
 	int count;
-	/* awful kludge to work around parse()'s pre-decrement */
+	struct trap *traphere = t_at(u.ux, u.uy);
+ 	/* awful kludge to work around parse()'s pre-decrement */
 	count = (multi || (save_cm && *save_cm == ',')) ? multi + 1 : 0;
 	multi = 0;	/* always reset */
 	/* uswallow case added by GAN 01/29/87 */
@@ -1690,13 +1883,27 @@ dopickup()
 	if (!can_reach_floor()) {
 #ifdef STEED
 		if (u.usteed && P_SKILL(P_RIDING) < P_BASIC)
-			You("aren't skilled enough to reach from %s.",
-					mon_nam(u.usteed));
+		    You("aren't skilled enough to reach from %s.",
+			y_monnam(u.usteed));
 		else
 #endif
 		You("cannot reach the %s.", surface(u.ux,u.uy));
 		return(0);
 	}
+
+ 	if (traphere && traphere->tseen) {
+		/* Allow pickup from holes and trap doors that you escaped from
+		 * because that stuff is teetering on the edge just like you, but
+		 * not pits, because there is an elevation discrepancy with stuff
+		 * in pits.
+		 */
+		if ((traphere->ttyp == PIT || traphere->ttyp == SPIKED_PIT) &&
+		     (!u.utrap || (u.utrap && u.utraptype != TT_PIT))) {
+			You("cannot reach the bottom of the pit.");
+			return(0);
+		}
+	}
+
 	return (pickup(-count));
 }
 
@@ -1744,7 +1951,10 @@ lookaround()
 	if (IS_ROCK(levl[x][y].typ) || (levl[x][y].typ == ROOM) ||
 	    IS_AIR(levl[x][y].typ))
 	    continue;
-	else if (closed_door(x,y)) {
+	else if (closed_door(x,y) ||
+		 (mtmp && mtmp->m_ap_type == M_AP_FURNITURE &&
+		  (mtmp->mappearance == S_hcdoor ||
+		   mtmp->mappearance == S_vcdoor))) {
 	    if(x != u.ux && y != u.uy) continue;
 	    if(flags.run != 1) goto stop;
 	    goto bcorr;
@@ -1861,11 +2071,12 @@ nomul(nval)
 	u.uinvulnerable = FALSE;	/* Kludge to avoid ctrl-C bug -dlc */
 	u.usleep = 0;
 	multi = nval;
-	flags.travel = flags.mv = flags.run = 0;
+	flags.travel = iflags.travel1 = flags.mv = flags.run = 0;
 }
 
 /* called when a non-movement, multi-turn action has completed */
-void unmul(msg_override)
+void
+unmul(msg_override)
 const char *msg_override;
 {
 	multi = 0;	/* caller will usually have done this already */
@@ -1948,7 +2159,7 @@ weight_cap()
 {
 	register long carrcap;
 
-	carrcap = (((ACURRSTR + ACURR(A_CON))/2)+1)*50;
+	carrcap = 25*(ACURRSTR + ACURR(A_CON)) + 50;
 	if (Upolyd) {
 		/* consistent with can_carry() in mon.c */
 		if (youmonst.data->mlet == S_NYMPH)
@@ -1992,14 +2203,14 @@ inv_weight()
 	   of invent for easier manipulation by askchain & co, but it's also
 	   retained in u.ugold in order to keep the status line accurate; we
 	   mustn't add its weight in twice under that circumstance */
-	wt = (otmp && otmp->oclass == GOLD_CLASS) ? 0 :
+	wt = (otmp && otmp->oclass == COIN_CLASS) ? 0 :
 		(int)((u.ugold + 50L) / 100L);
 #endif
 	while (otmp) {
 #ifndef GOLDOBJ
 		if (otmp->otyp != BOULDER || !throws_rocks(youmonst.data))
 #else
-		if (otmp->oclass == GOLD_CLASS)
+		if (otmp->oclass == COIN_CLASS)
 			wt += (int)(((long)otmp->quan + 50L) / 100L);
 		else if (otmp->otyp != BOULDER || !throws_rocks(youmonst.data))
 #endif
@@ -2075,12 +2286,13 @@ inv_cnt()
 /* Intended use is for your or some monsters inventory, */
 /* now that u.gold/m.gold is gone.*/
 /* Counting money in a container might be possible too. */
-long money_cnt(otmp)
+long
+money_cnt(otmp)
 struct obj *otmp;
 {
         while(otmp) {
 	        /* Must change when silver & copper is implemented: */
- 	        if (otmp->oclass == GOLD_CLASS) return otmp->quan;
+ 	        if (otmp->oclass == COIN_CLASS) return otmp->quan;
   	        otmp = otmp->nobj;
 	}
 	return 0;
