@@ -1,9 +1,8 @@
-/*	SCCS Id: @(#)mkobj.c	3.3	1999/02/13	*/
+/*	SCCS Id: @(#)mkobj.c	3.4	2002/10/07	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
-#include "artifact.h"
 #include "prop.h"
 
 STATIC_DCL void FDECL(mkbox_cnts,(struct obj *));
@@ -17,6 +16,8 @@ STATIC_DCL void FDECL(check_contained, (struct obj *,const char *));
 #endif
 #endif /* OVL1 */
 
+extern struct obj *thrownobj;		/* defined in dothrow.c */
+
 /*#define DEBUG_EFFECTS*/	/* show some messages for debugging */
 
 struct icp {
@@ -46,7 +47,7 @@ const struct icp boxiprobs[] = {
 {18, POTION_CLASS},
 {18, SCROLL_CLASS},
 {12, SPBOOK_CLASS},
-{ 7, GOLD_CLASS},
+{ 7, COIN_CLASS},
 { 6, WAND_CLASS},
 { 5, RING_CLASS},
 { 1, AMULET_CLASS}
@@ -78,26 +79,26 @@ const struct icp hellprobs[] = {
 };
 
 struct obj *
-mkobj_at(let,x,y, artif)
+mkobj_at(let, x, y, artif)
 char let;
-int x,y;
+int x, y;
 boolean artif;
 {
-	register struct obj *otmp;
+	struct obj *otmp;
 
-	otmp = mkobj(let,artif);
+	otmp = mkobj(let, artif);
 	place_object(otmp, x, y);
 	return(otmp);
 }
 
 struct obj *
-mksobj_at(otyp,x,y,init)
-int otyp,x,y;
-boolean init;
+mksobj_at(otyp, x, y, init, artif)
+int otyp, x, y;
+boolean init, artif;
 {
-	register struct obj *otmp;
+	struct obj *otmp;
 
-	otmp = mksobj(otyp,init,TRUE);
+	otmp = mksobj(otyp, init, artif);
 	place_object(otmp, x, y);
 	return(otmp);
 }
@@ -107,7 +108,7 @@ mkobj(oclass, artif)
 char oclass;
 boolean artif;
 {
-	register int tprob, i, prob = rnd(1000);
+	int tprob, i, prob = rnd(1000);
 
 	if(oclass == RANDOM_CLASS) {
 		const struct icp *iprobs =
@@ -138,21 +139,21 @@ mkbox_cnts(box)
 struct obj *box;
 {
 	register int n;
-	register struct obj *otmp, *gold = 0;
+	register struct obj *otmp;
 
 	box->cobj = (struct obj *) 0;
 
-	switch(box->otyp) {
-		case ICE_BOX:		n = 20; break;
-		case CHEST:		n = 5; break;
-		case LARGE_BOX:		n = 3; break;
-		case SACK:
-		case OILSKIN_SACK:
+	switch (box->otyp) {
+	case ICE_BOX:		n = 20; break;
+	case CHEST:		n = 5; break;
+	case LARGE_BOX:		n = 3; break;
+	case SACK:
+	case OILSKIN_SACK:
 				/* initial inventory: sack starts out empty */
 				if (moves <= 1 && !in_mklev) { n = 0; break; }
 				/*else FALLTHRU*/
-		case BAG_OF_HOLDING:	n = 1; break;
-		default:		n = 0; break;
+	case BAG_OF_HOLDING:	n = 1; break;
+	default:		n = 0; break;
 	}
 
 	for (n = rn2(n+1); n > 0; n--) {
@@ -175,16 +176,10 @@ struct obj *box;
 		if (!(otmp = mkobj(iprobs->iclass, TRUE))) continue;
 
 		/* handle a couple of special cases */
-		if (otmp->oclass == GOLD_CLASS) {
+		if (otmp->oclass == COIN_CLASS) {
 		    /* 2.5 x level's usual amount; weight adjusted below */
 		    otmp->quan = (long)(rnd(level_difficulty()+2) * rnd(75));
-		    if (gold) {			/* gold already in this box */
-			gold->quan += otmp->quan;	/* merge */
-			dealloc_obj(otmp);	/* note: not yet in any chain */
-			continue;
-		    } else {
-			gold = otmp;		/* remember this object */
-		    }
+		    otmp->owt = weight(otmp);
 		} else while (otmp->otyp == ROCK) {
 		    otmp->otyp = rnd_class(DILITHIUM_CRYSTAL, LOADSTONE);
 		    if (otmp->quan > 2L) otmp->quan = 1L;
@@ -199,10 +194,8 @@ struct obj *box;
 			    otmp->otyp = rnd_class(WAN_LIGHT, WAN_LIGHTNING);
 		}
 	    }
-	    add_to_container(box, otmp);
+	    (void) add_to_container(box, otmp);
 	}
-	if (gold) gold->owt = weight(gold);	/* quantity was diddled */
-	return;
 }
 
 int
@@ -225,10 +218,10 @@ rndmonnum()	/* select a random, common monster type */
 }
 
 /*
- * Split obj so that it gets size num. The remainder is put in the object
- * structure delivered by this call.  The object is positioned just
- * following the original in the nobj chain (and nexthere chain when on
- * the floor).
+ * Split obj so that it gets size gets reduced by num. The quantity num is
+ * put in the object structure delivered by this call.  The returned object
+ * has its wornmask cleared and is positioned just following the original
+ * in the nobj chain (and nexthere chain when on the floor).
  */
 struct obj *
 splitobj(obj, num)
@@ -237,7 +230,7 @@ long num;
 {
 	struct obj *otmp;
 
-	if (obj->cobj || num <= 0L || obj->quan < num)
+	if (obj->cobj || num <= 0L || obj->quan <= num)
 	    panic("splitobj");	/* can't split containers */
 	otmp = newobj(obj->oxlth + obj->onamelth);
 	*otmp = *obj;		/* copies whole structure */
@@ -245,9 +238,10 @@ long num;
 	if (!otmp->o_id) otmp->o_id = flags.ident++;	/* ident overflowed */
 	otmp->timed = 0;	/* not timed, yet */
 	otmp->lamplit = 0;	/* ditto */
-	obj->quan = num;
+	otmp->owornmask = 0L;	/* new object isn't worn */
+	obj->quan -= num;
 	obj->owt = weight(obj);
-	otmp->quan -= num;
+	otmp->quan = num;
 	otmp->owt = weight(otmp);	/* -= obj->owt ? */
 	obj->nobj = otmp;
 	/* Only set nexthere when on the floor, nexthere is also used */
@@ -265,6 +259,58 @@ long num;
 	return otmp;
 }
 
+/*
+ * Insert otmp right after obj in whatever chain(s) it is on.  Then extract
+ * obj from the chain(s).  This function does a literal swap.  It is up to
+ * the caller to provide a valid context for the swap.  When done, obj will
+ * still exist, but not on any chain.
+ *
+ * Note:  Don't use use obj_extract_self() -- we are doing an in-place swap,
+ * not actually moving something.
+ */
+void
+replace_object(obj, otmp)
+struct obj *obj;
+struct obj *otmp;
+{
+    otmp->where = obj->where;
+    switch (obj->where) {
+    case OBJ_FREE:
+	/* do nothing */
+	break;
+    case OBJ_INVENT:
+	otmp->nobj = obj->nobj;
+	obj->nobj = otmp;
+	extract_nobj(obj, &invent);
+	break;
+    case OBJ_CONTAINED:
+	otmp->nobj = obj->nobj;
+	otmp->ocontainer = obj->ocontainer;
+	obj->nobj = otmp;
+	extract_nobj(obj, &obj->ocontainer->cobj);
+	break;
+    case OBJ_MINVENT:
+	otmp->nobj = obj->nobj;
+	otmp->ocarry =  obj->ocarry;
+	obj->nobj = otmp;
+	extract_nobj(obj, &obj->ocarry->minvent);
+	break;
+    case OBJ_FLOOR:
+	otmp->nobj = obj->nobj;
+	otmp->nexthere = obj->nexthere;
+	otmp->ox = obj->ox;
+	otmp->oy = obj->oy;
+	obj->nobj = otmp;
+	obj->nexthere = otmp;
+	extract_nobj(obj, &fobj);
+	extract_nexthere(obj, &level.objects[obj->ox][obj->oy]);
+	break;
+    default:
+	panic("replace_object: obj position");
+	break;
+    }
+}
+
 /*
  * Create a dummy duplicate to put on shop bill.  The duplicate exists
  * only in the billobjs chain.  This function is used when a shop object
@@ -272,6 +318,12 @@ long num;
  * purposes.  For example, when eating, where an interruption will yield
  * an object which is different from what it started out as; the "I x"
  * command needs to display the original object.
+ *
+ * The caller is responsible for checking otmp->unpaid and
+ * costly_spot(u.ux, u.uy).  This function will make otmp no charge.
+ *
+ * Note that check_unpaid_usage() should be used instead for partial
+ * usage of an object.
  */
 void
 bill_dummy_object(otmp)
@@ -294,6 +346,9 @@ register struct obj *otmp;
 	    (void)strncpy(ONAME(dummy), ONAME(otmp), (int)otmp->onamelth);
 	if (Is_candle(dummy)) dummy->lamplit = 0;
 	addtobill(dummy, FALSE, TRUE, TRUE);
+	otmp->no_charge = 1;
+	otmp->unpaid = 0;
+	return;
 }
 
 #endif /* OVL1 */
@@ -361,7 +416,7 @@ boolean artif;
 		   this level; let's create an adventurer's corpse instead, then */
 			otmp->corpsenm = PM_HUMAN;
 		}
-		start_corpse_timeout(otmp);
+		/* timer set below */
 		break;
 	    case EGG:
 		otmp->corpsenm = NON_PM;	/* generic egg */
@@ -391,14 +446,17 @@ boolean artif;
 	    case SLIME_MOLD:
 		otmp->spe = current_fruit;
 		break;
+	    case KELP_FROND:
+		otmp->quan = (long) rnd(2);
+		break;
 	    }
-	    if (otmp->otyp == CORPSE || otmp->otyp == MEAT_RING) break;
+	    if (otmp->otyp == CORPSE || otmp->otyp == MEAT_RING ||
+		otmp->otyp == KELP_FROND) break;
 	    /* fall into next case */
 
 	case GEM_CLASS:
 		if (otmp->otyp == LOADSTONE) curse(otmp);
 		else if (otmp->otyp == ROCK) otmp->quan = (long) rn1(6,6);
-		else if (otmp->otyp == KELP_FROND) otmp->quan = (long) rnd(4);
 		else if (otmp->otyp != LUCKSTONE && !rn2(6)) otmp->quan = 2L;
 		else otmp->quan = 1L;
 		break;
@@ -466,6 +524,7 @@ boolean artif;
 	    }
 	    break;
 	case AMULET_CLASS:
+		if (otmp->otyp == AMULET_OF_YENDOR) flags.made_amulet = TRUE;
 		if(rn2(10) && (otmp->otyp == AMULET_OF_STRANGULATION ||
 		   otmp->otyp == AMULET_OF_CHANGE ||
 		   otmp->otyp == AMULET_OF_RESTFUL_SLEEP)) {
@@ -547,16 +606,25 @@ boolean artif;
 			otmp->corpsenm = rndmonnum();
 			if (!verysmall(&mons[otmp->corpsenm]) &&
 				rn2(level_difficulty()/2 + 10) > 10)
-			    add_to_container(otmp, mkobj(SPBOOK_CLASS,FALSE));
+			    (void) add_to_container(otmp,
+						    mkobj(SPBOOK_CLASS,FALSE));
 		}
 		break;
-	case GOLD_CLASS:
+	case COIN_CLASS:
 		break;	/* do nothing */
 	default:
 		impossible("impossible mkobj %d, sym '%c'.", otmp->otyp,
 						objects[otmp->otyp].oc_class);
 		return (struct obj *)0;
 	}
+
+	/* Some things must get done (timers) even if init = 0 */
+	switch (otmp->otyp) {
+	    case CORPSE:
+		start_corpse_timeout(otmp);
+		break;
+	}
+
 	/* unique objects may have an associated artifact entry */
 	if (objects[otyp].oc_unique && !otmp->oartifact)
 	    otmp = mk_artifact(otmp, (aligntyp)A_NONE);
@@ -565,27 +633,32 @@ boolean artif;
 }
 
 /*
- * Start a corpse decay or revive timer.  This assumes that the corpse
- * was just dropped and its age is 0.
+ * Start a corpse decay or revive timer.
+ * This takes the age of the corpse into consideration as of 3.4.0.
  */
 void
 start_corpse_timeout(body)
 	struct obj *body;
 {
-	long when;
+	long when; 		/* rot away when this old */
+	long corpse_age;	/* age of corpse          */
 	int rot_adjust;
 	short action;
 
 #define TAINT_AGE (50L)		/* age when corpses go bad */
 #define TROLL_REVIVE_CHANCE 37	/* 1/37 chance for 50 turns ~ 75% chance */
-#define ROT_AGE (250L)	/* age when corpses rot away */
+#define ROT_AGE (250L)		/* age when corpses rot away */
 
 	/* lizards and lichen don't rot or revive */
 	if (body->corpsenm == PM_LIZARD || body->corpsenm == PM_LICHEN) return;
 
 	action = ROT_CORPSE;		/* default action: rot away */
-	when = ROT_AGE;			/* rot away when this old */
 	rot_adjust = in_mklev ? 25 : 10;	/* give some variation */
+	corpse_age = monstermoves - body->age;
+	if (corpse_age > ROT_AGE)
+		when = rot_adjust;
+	else
+		when = ROT_AGE - corpse_age;
 	when += (long)(rnz(rot_adjust) - rot_adjust);
 
 	if (is_rider(&mons[body->corpsenm])) {
@@ -597,7 +670,7 @@ start_corpse_timeout(body)
 		for (when = 12L; when < 500L; when++)
 		    if (!rn2(3)) break;
 
-	} else if (mons[body->corpsenm].mlet == S_TROLL) {
+	} else if (mons[body->corpsenm].mlet == S_TROLL && !body->norevive) {
 		long age;
 		for (age = 2; age <= TAINT_AGE; age++)
 		    if (!rn2(TROLL_REVIVE_CHANCE)) {	/* troll revives */
@@ -606,7 +679,8 @@ start_corpse_timeout(body)
 			break;
 		    }
 	}
-
+	
+	if (body->norevive) body->norevive = 0;
 	(void) start_timer(when, TIMER_OBJECT, action, (genericptr_t)body);
 }
 
@@ -614,10 +688,12 @@ void
 bless(otmp)
 register struct obj *otmp;
 {
+#ifdef GOLDOBJ
+	if (otmp->oclass == COIN_CLASS) return;
+#endif
 	otmp->cursed = 0;
 	otmp->blessed = 1;
-	if (otmp->otyp == LUCKSTONE
-		|| (otmp->oartifact && spec_ability(otmp, SPFX_LUCK)))
+	if (carried(otmp) && confers_luck(otmp))
 	    set_moreluck();
 	else if (otmp->otyp == BAG_OF_HOLDING)
 	    otmp->owt = weight(otmp);
@@ -631,8 +707,7 @@ unbless(otmp)
 register struct obj *otmp;
 {
 	otmp->blessed = 0;
-	if (otmp->otyp == LUCKSTONE
-		|| (otmp->oartifact && spec_ability(otmp, SPFX_LUCK)))
+	if (carried(otmp) && confers_luck(otmp))
 	    set_moreluck();
 	else if (otmp->otyp == BAG_OF_HOLDING)
 	    otmp->owt = weight(otmp);
@@ -642,20 +717,29 @@ void
 curse(otmp)
 register struct obj *otmp;
 {
+#ifdef GOLDOBJ
+	if (otmp->oclass == COIN_CLASS) return;
+#endif
 	otmp->blessed = 0;
 	otmp->cursed = 1;
-	if (otmp->otyp == LUCKSTONE
-		|| (otmp->oartifact && spec_ability(otmp, SPFX_LUCK)))
+	/* welded two-handed weapon interferes with some armor removal */
+	if (otmp == uwep && bimanual(uwep)) reset_remarm();
+	/* rules at top of wield.c state that twoweapon cannot be done
+	   with cursed alternate weapon */
+	if (otmp == uswapwep && u.twoweap)
+	    drop_uswapwep();
+	/* some cursed items need immediate updating */
+	if (carried(otmp) && confers_luck(otmp))
 	    set_moreluck();
 	else if (otmp->otyp == BAG_OF_HOLDING)
 	    otmp->owt = weight(otmp);
 	else if (otmp->otyp == FIGURINE) {
 		if (otmp->corpsenm != NON_PM
-	    	    && !dead_species(otmp->corpsenm,TRUE)
+		    && !dead_species(otmp->corpsenm,TRUE)
 		    && (carried(otmp) || mcarried(otmp)))
 			attach_fig_transform_timeout(otmp);
 	}
- 	return;
+	return;
 }
 
 void
@@ -663,13 +747,12 @@ uncurse(otmp)
 register struct obj *otmp;
 {
 	otmp->cursed = 0;
-	if (otmp->otyp == LUCKSTONE
-		|| (otmp->oartifact && spec_ability(otmp, SPFX_LUCK)))
+	if (carried(otmp) && confers_luck(otmp))
 	    set_moreluck();
 	else if (otmp->otyp == BAG_OF_HOLDING)
-		otmp->owt = weight(otmp);
+	    otmp->owt = weight(otmp);
 	else if (otmp->otyp == FIGURINE && otmp->timed)
-		(void) stop_timer(FIG_TRANSFORM, (genericptr_t) otmp);
+	    (void) stop_timer(FIG_TRANSFORM, (genericptr_t) otmp);
 	return;
 }
 
@@ -683,12 +766,13 @@ register int chance;
 {
 	if(otmp->blessed || otmp->cursed) return;
 
-	if(!rn2(chance))
+	if(!rn2(chance)) {
 	    if(!rn2(2)) {
 		curse(otmp);
 	    } else {
 		bless(otmp);
 	    }
+	}
 	return;
 }
 
@@ -756,9 +840,15 @@ register struct obj *obj;
 
 		return wt + cwt;
 	}
-	if (obj->otyp == CORPSE && obj->corpsenm >= LOW_PM)
-		return (int)obj->quan * mons[obj->corpsenm].cwt;
-	else if (obj->oclass == GOLD_CLASS)
+	if (obj->otyp == CORPSE && obj->corpsenm >= LOW_PM) {
+		long long_wt = obj->quan * (long) mons[obj->corpsenm].cwt;
+
+		wt = (long_wt > LARGEST_INT) ? LARGEST_INT : (int)long_wt;
+		if (obj->oeaten) wt = eaten_stat(wt, obj);
+		return wt;
+	} else if (obj->oclass == FOOD_CLASS && obj->oeaten) {
+		return eaten_stat((int)obj->quan * wt, obj);
+	} else if (obj->oclass == COIN_CLASS)
 		return (int)((obj->quan + 50L) / 100L);
 	else if (obj->otyp == HEAVY_IRON_BALL && obj->owt != 0)
 		return((int)(obj->owt));	/* kludge for "very" heavy iron ball */
@@ -769,8 +859,9 @@ static int treefruits[] = {APPLE,ORANGE,PEAR,BANANA,EUCALYPTUS_LEAF};
 
 struct obj *
 rnd_treefruit_at(x,y)
+int x, y;
 {
-	return mksobj_at(treefruits[rn2(SIZE(treefruits)-1)],x,y,TRUE);
+	return mksobj_at(treefruits[rn2(SIZE(treefruits))], x, y, TRUE, FALSE);
 }
 #endif /* OVL0 */
 #ifdef OVLB
@@ -782,11 +873,12 @@ int x, y;
 {
     register struct obj *gold = g_at(x,y);
 
-    if (amount <= 0L) amount = (long)(1 + rnd(level_difficulty()+2) * rnd(30));
+    if (amount <= 0L)
+	amount = (long)(1 + rnd(level_difficulty()+2) * rnd(30));
     if (gold) {
 	gold->quan += amount;
     } else {
-	gold = mksobj_at(GOLD_PIECE,x,y,TRUE);
+	gold = mksobj_at(GOLD_PIECE, x, y, TRUE, FALSE);
 	gold->quan = amount;
     }
     gold->owt = weight(gold);
@@ -823,7 +915,11 @@ boolean init;
 
 	if (objtype != CORPSE && objtype != STATUE)
 	    impossible("making corpstat type %d", objtype);
-	otmp = mksobj_at(objtype, x, y, init);
+	if (x == 0 && y == 0) {		/* special case - random placement */
+		otmp = mksobj(objtype, init, FALSE);
+		if (otmp) rloco(otmp);
+	} else
+		otmp = mksobj_at(objtype, x, y, init, FALSE);
 	if (otmp) {
 	    if (mtmp) {
 		struct obj *otmp2;
@@ -851,6 +947,33 @@ boolean init;
 	return(otmp);
 }
 
+/*
+ * Attach a monster id to an object, to provide
+ * a lasting association between the two.
+ */
+struct obj *
+obj_attach_mid(obj, mid)
+struct obj *obj;
+unsigned mid;
+{
+    struct obj *otmp;
+    int lth, namelth;
+
+    if (!mid || !obj) return (struct obj *)0;
+    lth = sizeof(mid);
+    namelth = obj->onamelth ? strlen(ONAME(obj)) + 1 : 0;
+    if (namelth) 
+	otmp = realloc_obj(obj, lth, (genericptr_t) &mid, namelth, ONAME(obj));
+    else {
+	otmp = obj;
+	otmp->oxlth = sizeof(mid);
+	(void) memcpy((genericptr_t)otmp->oextra, (genericptr_t)&mid,
+								sizeof(mid));
+    }
+    if (otmp && otmp->oxlth) otmp->oattached = OATTACHED_M_ID;	/* mark it */
+    return otmp;
+}
+
 static struct obj *
 save_mtraits(obj, mtmp)
 struct obj *obj;
@@ -865,8 +988,9 @@ struct monst *mtmp;
 	if (otmp && otmp->oxlth) {
 		struct monst *mtmp2 = (struct monst *)otmp->oextra;
 		if (mtmp->data) mtmp2->mnum = monsndx(mtmp->data);
-		/* invalidate pointers and m_id */
-		mtmp2->m_id     = 0;
+		/* invalidate pointers */
+		/* m_id is needed to know if this is a revived quest leader */
+		/* but m_id must be cleared when loading bones */
 		mtmp2->nmon     = (struct monst *)0;
 		mtmp2->data     = (struct permonst *)0;
 		mtmp2->minvent  = (struct obj *)0;
@@ -917,7 +1041,7 @@ register int x, y;
 
 	/* player statues never contain books */
 	initialize_it = (objtype != STATUE);
-	if ((otmp = mksobj_at(objtype, x, y, initialize_it)) != 0) {
+	if ((otmp = mksobj_at(objtype, x, y, initialize_it, FALSE)) != 0) {
 	    /* tt_oname will return null if the scoreboard is empty */
 	    if ((otmp2 = tt_oname(otmp)) != 0) otmp = otmp2;
 	}
@@ -946,8 +1070,19 @@ is_flammable(otmp)
 register struct obj *otmp;
 {
 	int otyp = otmp->otyp;
+	int omat = objects[otyp].oc_material;
 
-	if (objects[otyp].oc_oprop == FIRE_RES) return FALSE;
+	if (objects[otyp].oc_oprop == FIRE_RES || otyp == WAN_FIRE)
+		return FALSE;
+
+	return((boolean)((omat <= WOOD && omat != LIQUID) || omat == PLASTIC));
+}
+
+boolean
+is_rottable(otmp)
+register struct obj *otmp;
+{
+	int otyp = otmp->otyp;
 
 	return((boolean)(objects[otyp].oc_material <= WOOD &&
 			objects[otyp].oc_material != LIQUID));
@@ -972,6 +1107,7 @@ int x, y;
     if (otmp->where != OBJ_FREE)
 	panic("place_object: obj not free");
 
+    obj_no_longer_held(otmp);
     if (otmp->otyp == BOULDER) block_point(x,y);	/* vision */
 
     /* obj goes under boulders */
@@ -1244,7 +1380,12 @@ extract_nexthere(obj, head_ptr)
 }
 
 
-void
+/*
+ * Add obj to mon's inventory.  If obj is able to merge with something already
+ * in the inventory, then the passed obj is deleted and 1 is returned.
+ * Otherwise 0 is returned.
+ */
+int
 add_to_minv(mon, obj)
     struct monst *mon;
     struct obj *obj;
@@ -1257,25 +1398,39 @@ add_to_minv(mon, obj)
     /* merge if possible */
     for (otmp = mon->minvent; otmp; otmp = otmp->nobj)
 	if (merged(&otmp, &obj))
-	    return;
+	    return 1;	/* obj merged and then free'd */
     /* else insert; don't bother forcing it to end of chain */
     obj->where = OBJ_MINVENT;
     obj->ocarry = mon;
     obj->nobj = mon->minvent;
     mon->minvent = obj;
+    return 0;	/* obj on mon's inventory chain */
 }
 
-void
+/*
+ * Add obj to container, make sure obj is "free".  Returns (merged) obj.
+ * The input obj may be deleted in the process.
+ */
+struct obj *
 add_to_container(container, obj)
     struct obj *container, *obj;
 {
+    struct obj *otmp;
+
     if (obj->where != OBJ_FREE)
 	panic("add_to_container: obj not free");
+    if (container->where != OBJ_INVENT && container->where != OBJ_MINVENT)
+	obj_no_longer_held(obj);
+
+    /* merge if possible */
+    for (otmp = container->cobj; otmp; otmp = otmp->nobj)
+	if (merged(&otmp, &obj)) return (otmp);
 
     obj->where = OBJ_CONTAINED;
     obj->ocontainer = container;
     obj->nobj = container->cobj;
     container->cobj = obj;
+    return (obj);
 }
 
 void
@@ -1342,6 +1497,8 @@ dealloc_obj(obj)
     if (obj_sheds_light(obj))
 	del_light_source(LS_OBJECT, (genericptr_t) obj);
 
+    if (obj == thrownobj) thrownobj = (struct obj*)0;
+
     free((genericptr_t) obj);
 }
 
@@ -1417,9 +1574,9 @@ obj_sanity_check()
 	}
 	/* shouldn't be a full container on the bill */
 	if (obj->cobj) {
-	    pline("%s obj %s contains something! %s\n", mesg,
+	    pline("%s obj %s contains %s! %s\n", mesg,
 		fmt_ptr((genericptr_t)obj, obj_address),
-		doname(obj));
+		something, doname(obj));
 	}
     }
 
