@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)pager.c	3.3	1999/10/10	*/
+/*	SCCS Id: @(#)pager.c	3.4	2003/08/13	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -65,7 +65,7 @@ lookat(x, y, buf, monbuf)
 
     buf[0] = monbuf[0] = 0;
     glyph = glyph_at(x,y);
-    if (u.ux == x && u.uy == y && canseeself()) {
+    if (u.ux == x && u.uy == y && senseself()) {
 	char race[QBUFSZ];
 
 	/* if not polymorphed, show both the role and the race */
@@ -79,6 +79,11 @@ lookat(x, y, buf, monbuf)
 		race,
 		mons[u.umonnum].mname,
 		plname);
+	/* file lookup can't distinguish between "gnomish wizard" monster
+	   and correspondingly named player character, always picking the
+	   former; force it to find the general "wizard" entry instead */
+	if (Role_if(PM_WIZARD) && Race_if(PM_GNOME) && !Upolyd)
+	    pm = &mons[PM_WIZARD];
 
 #ifdef STEED
 	if (u.usteed) {
@@ -89,6 +94,27 @@ lookat(x, y, buf, monbuf)
 	    Strcat(buf, steedbuf);
 	}
 #endif
+	/* When you see yourself normally, no explanation is appended
+	   (even if you could also see yourself via other means).
+	   Sensing self while blind or swallowed is treated as if it
+	   were by normal vision (cf canseeself()). */
+	if ((Invisible || u.uundetected) && !Blind && !u.uswallow) {
+	    unsigned how = 0;
+
+	    if (Infravision)	 how |= 1;
+	    if (Unblind_telepat) how |= 2;
+	    if (Detect_monsters) how |= 4;
+
+	    if (how)
+		Sprintf(eos(buf), " [seen: %s%s%s%s%s]",
+			(how & 1) ? "infravision" : "",
+			/* add comma if telep and infrav */
+			((how & 3) > 2) ? ", " : "",
+			(how & 2) ? "telepathy" : "",
+			/* add comma if detect and (infrav or telep or both) */
+			((how & 7) > 4) ? ", " : "",
+			(how & 4) ? "monster detection" : "");
+	}
     } else if (u.uswallow) {
 	/* all locations when swallowed other than the hero are the monster */
 	Sprintf(buf, "interior of %s",
@@ -98,24 +124,40 @@ lookat(x, y, buf, monbuf)
 	bhitpos.x = x;
 	bhitpos.y = y;
 	mtmp = m_at(x,y);
-	if(mtmp != (struct monst *) 0) {
-	    register boolean hp = (mtmp->data == &mons[PM_HIGH_PRIEST]);
+	if (mtmp != (struct monst *) 0) {
+	    char *name, monnambuf[BUFSZ];
+	    boolean accurate = !Hallucination;
+
+	    if (mtmp->data == &mons[PM_COYOTE] && accurate)
+		name = coyotename(mtmp, monnambuf);
+	    else
+		name = distant_monnam(mtmp, ARTICLE_NONE, monnambuf);
 
 	    pm = mtmp->data;
 	    Sprintf(buf, "%s%s%s",
 		    (mtmp->mx != x || mtmp->my != y) ?
-			((mtmp->isshk && !Hallucination)
+			((mtmp->isshk && accurate)
 				? "tail of " : "tail of a ") : "",
-		    (!hp && mtmp->mtame && !Hallucination) ? "tame " :
-		    (!hp && mtmp->mpeaceful && !Hallucination) ?
-		                                          "peaceful " : "",
-		    (hp ? "high priest" : l_monnam(mtmp)));
+		    (mtmp->mtame && accurate) ? "tame " :
+		    (mtmp->mpeaceful && accurate) ? "peaceful " : "",
+		    name);
 	    if (u.ustuck == mtmp)
 		Strcat(buf, (Upolyd && sticks(youmonst.data)) ?
 			", being held" : ", holding you");
 	    if (mtmp->mleashed)
 		Strcat(buf, ", leashed to you");
 
+	    if (mtmp->mtrapped && cansee(mtmp->mx, mtmp->my)) {
+		struct trap *t = t_at(mtmp->mx, mtmp->my);
+		int tt = t ? t->ttyp : NO_TRAP;
+
+		/* newsym lets you know of the trap, so mention it here */
+		if (tt == BEAR_TRAP || tt == PIT ||
+			tt == SPIKED_PIT || tt == WEB)
+		    Sprintf(eos(buf), ", trapped in %s",
+			    an(defsyms[trap_to_defsym(tt)].explanation));
+	    }
+
 	    {
 		int ways_seen = 0, normal = 0, xraydist;
 		boolean useemon = (boolean) canseemon(mtmp);
@@ -142,6 +184,8 @@ lookat(x, y, buf, monbuf)
 		    ways_seen++;
 		if (Detect_monsters)
 		    ways_seen++;
+		if (MATCH_WARN_OF_MON(mtmp))
+		    ways_seen++;
 
 		if (ways_seen > 1 || !normal) {
 		    if (normal) {
@@ -172,6 +216,17 @@ lookat(x, y, buf, monbuf)
 			Strcat(monbuf, "monster detection");
 			if (ways_seen-- > 1) Strcat(monbuf, ", ");
 		    }
+		    if (MATCH_WARN_OF_MON(mtmp)) {
+		    	char wbuf[BUFSZ];
+			if (Hallucination)
+				Strcat(monbuf, "paranoid delusion");
+			else {
+				Sprintf(wbuf, "warned of %s",
+					makeplural(mtmp->data->mname));
+		    		Strcat(monbuf, wbuf);
+		    	}
+		    	if (ways_seen-- > 1) Strcat(monbuf, ", ");
+		    }
 		}
 	    }
 	}
@@ -182,7 +237,7 @@ lookat(x, y, buf, monbuf)
 	if (!otmp || otmp->otyp != glyph_to_obj(glyph)) {
 	    if (glyph_to_obj(glyph) != STRANGE_OBJECT) {
 		otmp = mksobj(glyph_to_obj(glyph), FALSE, FALSE);
-		if (otmp->oclass == GOLD_CLASS)
+		if (otmp->oclass == COIN_CLASS)
 		    otmp->quan = 2L; /* to force pluralization */
 		else if (otmp->otyp == SLIME_MOLD)
 		    otmp->spe = current_fruit;	/* give the fruit a type */
@@ -294,12 +349,13 @@ checkfile(inp, pm, user_typed_name, without_asking)
     if (*dbase_str) {
 	/* adjust the input to remove "named " and convert to lower case */
 	char *alt = 0;	/* alternate description */
+
 	if ((ep = strstri(dbase_str, " named ")) != 0)
 	    alt = ep + 7;
 	else
 	    ep = strstri(dbase_str, " called ");
-	if (ep) *ep = '\0';
-	else if ((ep = strstri(dbase_str, ", ")) != 0) *ep = '\0';
+	if (!ep) ep = strstri(dbase_str, ", ");
+	if (ep && ep > dbase_str) *ep = '\0';
 
 	/*
 	 * If the object is named, then the name is the alternate description;
@@ -413,7 +469,7 @@ do_look(quick)
     boolean from_screen;	/* question from the screen */
     boolean need_to_look;	/* need to get explan. from glyph */
     boolean hit_trap;		/* true if found trap explanation */
-    int skipped_venom = 0;	/* non-zero if we ignored "splash of venom" */
+    int skipped_venom;		/* non-zero if we ignored "splash of venom" */
     static const char *mon_interior = "the interior of a monster";
 
     if (quick) {
@@ -450,6 +506,7 @@ do_look(quick)
 	/* Reset some variables. */
 	need_to_look = FALSE;
 	pm = (struct permonst *)0;
+	skipped_venom = 0;
 	found = 0;
 	out_str[0] = '\0';
 
@@ -477,6 +534,8 @@ do_look(quick)
 		sym = showsyms[trap_to_defsym(glyph_to_trap(glyph))];
 	    } else if (glyph_is_object(glyph)) {
 		sym = oc_syms[(int)objects[glyph_to_obj(glyph)].oc_class];
+		if (sym == '`' && iflags.bouldersym && (int)glyph_to_obj(glyph) == BOULDER)
+			sym = iflags.bouldersym;
 	    } else if (glyph_is_monster(glyph)) {
 		/* takes care of pets, detected, ridden, and regular mons */
 		sym = monsyms[(int)mons[glyph_to_mon(glyph)].mlet];
@@ -484,6 +543,9 @@ do_look(quick)
 		sym = showsyms[glyph_to_swallow(glyph)+S_sw_tl];
 	    } else if (glyph_is_invisible(glyph)) {
 		sym = DEF_INVISIBLE;
+	    } else if (glyph_is_warning(glyph)) {
+		sym = glyph_to_warning(glyph);
+	    	sym = warnsyms[sym];
 	    } else {
 		impossible("do_look:  bad glyph %d at (%d,%d)",
 						glyph, (int)cc.x, (int)cc.y);
@@ -498,7 +560,8 @@ do_look(quick)
 
 	/* Check for monsters */
 	for (i = 0; i < MAXMCLASSES; i++) {
-	    if (sym == (from_screen ? monsyms[i] : def_monsyms[i])) {
+	    if (sym == (from_screen ? monsyms[i] : def_monsyms[i]) &&
+		monexplain[i]) {
 		need_to_look = TRUE;
 		if (!found) {
 		    Sprintf(out_str, "%c       %s", sym, an(monexplain[i]));
@@ -509,11 +572,14 @@ do_look(quick)
 		}
 	    }
 	}
-	/* handle '@' as a special case; firstmatch is guaranteed
-	   to already be set in that case */
-	if (!from_screen ? (sym == def_monsyms[S_HUMAN]) :
-		(cc.x == u.ux && cc.y == u.uy && sym == monsyms[S_HUMAN]))
-	     found += append_str(out_str, "you");	/* tack on "or you" */
+	/* handle '@' as a special case if it refers to you and you're
+	   playing a character which isn't normally displayed by that
+	   symbol; firstmatch is assumed to already be set for '@' */
+	if ((from_screen ?
+		(sym == monsyms[S_HUMAN] && cc.x == u.ux && cc.y == u.uy) :
+		(sym == def_monsyms[S_HUMAN] && !iflags.showrace)) &&
+	    !(Race_if(PM_HUMAN) || Race_if(PM_ELF)) && !Upolyd)
+	    found += append_str(out_str, "you");	/* tack on "or you" */
 
 	/*
 	 * Special case: if identifying from the screen, and we're swallowed,
@@ -594,6 +660,26 @@ do_look(quick)
 	    }
 	}
 
+	/* Now check for warning symbols */
+	for (i = 1; i < WARNCOUNT; i++) {
+	    x_str = def_warnsyms[i].explanation;
+	    if (sym == (from_screen ? warnsyms[i] : def_warnsyms[i].sym)) {
+		if (!found) {
+			Sprintf(out_str, "%c       %s",
+				sym, def_warnsyms[i].explanation);
+			firstmatch = def_warnsyms[i].explanation;
+			found++;
+		} else {
+			found += append_str(out_str, def_warnsyms[i].explanation);
+		}
+		/* Kludge: warning trumps boulders on the display.
+		   Reveal the boulder too or player can get confused */
+		if (from_screen && sobj_at(BOULDER, cc.x, cc.y))
+			Strcat(out_str, " co-located with a boulder");
+		break;	/* out of for loop*/
+	    }
+	}
+    
 	/* if we ignored venom and list turned out to be short, put it back */
 	if (skipped_venom && found < 2) {
 	    x_str = objexplain[VENOM_CLASS];
@@ -606,6 +692,17 @@ do_look(quick)
 	    }
 	}
 
+	/* handle optional boulder symbol as a special case */ 
+	if (iflags.bouldersym && sym == iflags.bouldersym) {
+	    if (!found) {
+		firstmatch = "boulder";
+		Sprintf(out_str, "%c       %s", sym, an(firstmatch));
+		found++;
+	    } else {
+		found += append_str(out_str, "boulder");
+	    }
+	}
+	
 	/*
 	 * If we are looking at the screen, follow multiple possibilities or
 	 * an ambiguous explanation by something more detailed.
@@ -613,14 +710,12 @@ do_look(quick)
 	if (from_screen) {
 	    if (found > 1 || need_to_look) {
 		char monbuf[BUFSZ];
-		char temp_buf[BUFSZ], coybuf[QBUFSZ];
+		char temp_buf[BUFSZ];
 
 		pm = lookat(cc.x, cc.y, look_buf, monbuf);
 		firstmatch = look_buf;
 		if (*firstmatch) {
-		    Sprintf(temp_buf, " (%s)",
-				(pm == &mons[PM_COYOTE]) ?
-				coyotename(coybuf) : firstmatch);
+		    Sprintf(temp_buf, " (%s)", firstmatch);
 		    (void)strncat(out_str, temp_buf, BUFSZ-strlen(out_str)-1);
 		    found = 1;	/* we have something to look up */
 		}
@@ -670,7 +765,7 @@ doidtrap()
 	register struct trap *trap;
 	int x, y, tt;
 
-	if (!getdir((char *)0)) return 0;
+	if (!getdir("^")) return 0;
 	x = u.ux + u.dx;
 	y = u.uy + u.dy;
 	for (trap = ftrap; trap; trap = trap->ntrap)
@@ -696,12 +791,14 @@ doidtrap()
 	return 0;
 }
 
-int
-dowhatdoes()
+char *
+dowhatdoes_core(q, cbuf)
+char q;
+char *cbuf;
 {
 	dlb *fp;
-	char bufr[BUFSZ+6];
-	register char *buf = &bufr[6], *ep, q, ctrl, meta;
+	char bufr[BUFSZ];
+	register char *buf = &bufr[6], *ep, ctrl, meta;
 
 	fp = dlb_fopen(CMDHELPFILE, "r");
 	if (!fp) {
@@ -709,16 +806,9 @@ dowhatdoes()
 		return 0;
 	}
 
-#if defined(UNIX) || defined(VMS)
-	introff();
-#endif
-	q = yn_function("What command?", (char *)0, '\0');
-#if defined(UNIX) || defined(VMS)
-	intron();
-#endif
-	ctrl = ((q <= '\033') ? (q - 1 + 'A') : 0);
+  	ctrl = ((q <= '\033') ? (q - 1 + 'A') : 0);
 	meta = ((0x80 & q) ? (0x7f & q) : 0);
-	while(dlb_fgets(buf,BUFSZ,fp))
+	while(dlb_fgets(buf,BUFSZ-6,fp)) {
 	    if ((ctrl && *buf=='^' && *(buf+1)==ctrl) ||
 		(meta && *buf=='M' && *(buf+1)=='-' && *(buf+2)==meta) ||
 		*buf==q) {
@@ -737,12 +827,33 @@ dowhatdoes()
 			buf[0] = q;
 			(void) strncpy(buf+1, "       ", 7);
 		}
-		pline("%s", buf);
 		(void) dlb_fclose(fp);
-		return 0;
+		Strcpy(cbuf, buf);
+		return cbuf;
 	    }
-	pline("I've never heard of such commands.");
+	}
 	(void) dlb_fclose(fp);
+	return (char *)0;
+}
+
+int
+dowhatdoes()
+{
+	char bufr[BUFSZ];
+	char q, *reslt;
+
+#if defined(UNIX) || defined(VMS)
+	introff();
+#endif
+	q = yn_function("What command?", (char *)0, '\0');
+#if defined(UNIX) || defined(VMS)
+	intron();
+#endif
+	reslt = dowhatdoes_core(q, bufr);
+	if (reslt)
+		pline("%s", reslt);
+	else
+		pline("I've never heard of such commands.");
 	return 0;
 }
 
