@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)polyself.c	3.4	2002/01/15	*/
+/*	SCCS Id: @(#)polyself.c	3.4	2003/01/08	*/
 /*	Copyright (C) 1987, 1988, 1989 by Ken Arromdee */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -33,8 +33,8 @@ polyman(fmt, arg)
 const char *fmt, *arg;
 {
 	boolean sticky = sticks(youmonst.data) && u.ustuck && !u.uswallow,
-		was_mimicking_gold = (youmonst.m_ap_type == M_AP_OBJECT
-				      && youmonst.mappearance == GOLD_PIECE);
+		was_mimicking = (youmonst.m_ap_type == M_AP_OBJECT);
+	boolean could_pass_walls = Passes_walls;
 	boolean was_blind = !!Blind;
 
 	if (Upolyd) {
@@ -52,19 +52,11 @@ const char *fmt, *arg;
 
 	if (sticky) uunstick();
 	find_ac();
-	if (was_mimicking_gold) {
+	if (was_mimicking) {
 	    if (multi < 0) unmul("");
-	} else {
-	    /*
-	     * Clear any in-progress imitations -- the case where not a
-	     * mimic is handled above.
-	     *
-	     * Except, this is not complete if the hero ever gets the
-	     * chance to imitate anything, then s/he may be mimicing
-	     * gold, but not the way its done for eating a mimic.
-	     */
 	    youmonst.m_ap_type = M_AP_NOTHING;
 	}
+
 	newsym(u.ux,u.uy);
 
 	You(fmt, arg);
@@ -83,6 +75,15 @@ const char *fmt, *arg;
 	    }
 	    done(GENOCIDED);
 	}
+
+	if (u.twoweap && !could_twoweap(youmonst.data))
+	    untwoweapon();
+
+	if (u.utraptype == TT_PIT) {
+	    if (could_pass_walls) {	/* player forms cannot pass walls */
+		u.utrap = rn1(6,2);
+	    }
+	}
 	if (was_blind && !Blind) {	/* reverting from eyeless */
 	    Blinded = 1L;
 	    make_blinded(0L, TRUE);	/* remove blindness */
@@ -155,7 +156,7 @@ newman()
 	reset_rndmonst(NON_PM);	/* new monster generation criteria */
 
 	/* random experience points for the new experience level */
-	u.uexp = rndexp();
+	u.uexp = rndexp(FALSE);
 
 	/* u.uhpmax * u.ulevel / oldlvl: proportionate hit points to new level
 	 * -10 and +10: don't apply proportionate HP to 10 of a starting
@@ -229,6 +230,7 @@ boolean forcecontrol;
 				uarm->otyp <= YELLOW_DRAGON_SCALES);
 	boolean iswere = (u.ulycn >= LOW_PM || is_were(youmonst.data));
 	boolean isvamp = (youmonst.data->mlet == S_VAMPIRE || u.umonnum == PM_VAMPIRE_BAT);
+	boolean was_floating = (Levitation || Flying);
 
         if(!Polymorph_control && !forcecontrol && !draconian && !iswere && !isvamp) {
 	    if (rn2(20) > ACURR(A_CON)) {
@@ -302,7 +304,7 @@ boolean forcecontrol;
 	/* The below polyok() fails either if everything is genocided, or if
 	 * we deliberately chose something illegal to force newman().
 	 */
-	if (!polyok(&mons[mntmp]) || !rn2(5))
+	if (!polyok(&mons[mntmp]) || !rn2(5) || your_race(&mons[mntmp]))
 		newman();
 	else if(!polymon(mntmp)) return;
 
@@ -318,6 +320,9 @@ boolean forcecontrol;
 		new_light_source(u.ux, u.uy, new_light,
 				 LS_MONSTER, (genericptr_t)&youmonst);
 	}
+	if (is_pool(u.ux,u.uy) && was_floating && !(Levitation || Flying) &&
+		!breathless(youmonst.data) && !amphibious(youmonst.data) &&
+		!Swimming) drown();
 }
 
 /* (try to) make a mntmp monster out of the player */
@@ -327,6 +332,7 @@ int	mntmp;
 {
 	boolean sticky = sticks(youmonst.data) && u.ustuck && !u.uswallow,
 		was_blind = !!Blind, dochange = FALSE;
+	boolean could_pass_walls = Passes_walls;
 	int mlvl;
 
 	if (mvitals[mntmp].mvflags & G_GENOD) {	/* allow G_EXTINCT */
@@ -352,10 +358,12 @@ int	mntmp;
 		flags.female = u.mfemale;
 	}
 
-	if (youmonst.m_ap_type == M_AP_OBJECT &&
-		youmonst.mappearance == GOLD_PIECE) {
-	    /* stop mimicking gold immediately */
+	if (youmonst.m_ap_type) {
+	    /* stop mimicking immediately */
 	    if (multi < 0) unmul("");
+	} else if (mons[mntmp].mlet != S_MIMIC) {
+	    /* as in polyman() */
+	    youmonst.m_ap_type = M_AP_NOTHING;
 	}
 	if (is_male(&mons[mntmp])) {
 		if(flags.female) dochange = TRUE;
@@ -404,7 +412,7 @@ int	mntmp;
 		You("no longer feel sick.");
 	}
 	if (Slimed) {
-	    if (mntmp == PM_FIRE_VORTEX || mntmp == PM_FIRE_ELEMENTAL || mntmp == PM_SALAMANDER) {
+	    if (flaming(youmonst.data)) {
 		pline_The("slime burns away!");
 		Slimed = 0L;
 		flags.botl = 1;
@@ -456,6 +464,13 @@ int	mntmp;
 	else
 		u.uundetected = 0;
 
+	if (u.utraptype == TT_PIT) {
+	    if (could_pass_walls && !Passes_walls) {
+		u.utrap = rn1(6,2);
+	    } else if (!could_pass_walls && Passes_walls) {
+		u.utrap = 0;
+	    }
+	}
 	if (was_blind && !Blind) {	/* previous form was eyeless */
 	    Blinded = 1L;
 	    make_blinded(0L, TRUE);	/* remove blindness */
@@ -583,7 +598,7 @@ break_armor()
 	}
 #endif
     } else if (sliparm(youmonst.data)) {
-	if ((otmp = uarm) != 0) {
+	if (((otmp = uarm) != 0) && (racial_exception(&youmonst, otmp) < 1)) {
 		if (donning(otmp)) cancel_don();
 		Your("armor falls around you!");
 		(void) Armor_gone();
@@ -606,6 +621,23 @@ break_armor()
 	}
 #endif
     }
+    if (has_horns(youmonst.data)) {
+	if ((otmp = uarmh) != 0) {
+	    if (is_flimsy(otmp) && !donning(otmp)) {
+		char hornbuf[BUFSZ], yourbuf[BUFSZ];
+
+		/* Future possiblities: This could damage/destroy helmet */
+		Sprintf(hornbuf, "horn%s", plur(num_horns(youmonst.data)));
+		Your("%s %s through %s %s.", hornbuf, vtense(hornbuf, "pierce"),
+		     shk_your(yourbuf, otmp), xname(otmp));
+	    } else {
+		if (donning(otmp)) cancel_don();
+		Your("helmet falls to the %s!", surface(u.ux, u.uy));
+		(void) Helmet_off();
+		dropx(otmp);
+	    }
+	}
+    }
     if (nohands(youmonst.data) || verysmall(youmonst.data)) {
 	if ((otmp = uarmg) != 0) {
 	    if (donning(otmp)) cancel_don();
@@ -668,6 +700,8 @@ int alone;
 		    dropx(otmp2);
 	    }
 	    untwoweapon();
+	} else if (!could_twoweap(youmonst.data)) {
+	    untwoweapon();
 	}
     }
 }
@@ -705,6 +739,8 @@ rehumanize()
 int
 dobreathe()
 {
+	struct attack *mattk;
+
 	if (Strangled) {
 	    You_cant("breathe.  Sorry.");
 	    return(0);
@@ -717,17 +753,13 @@ dobreathe()
 	flags.botl = 1;
 
 	if (!getdir((char *)0)) return(0);
-	else {
-	    register struct attack *mattk;
-	    register int i;
 
-	    for(i = 0; i < NATTK; i++) {
-		mattk = &(youmonst.data->mattk[i]);
-		if(mattk->aatyp == AT_BREA) break;
-	    }
+	mattk = attacktype_fordmg(youmonst.data, AT_BREA, AD_ANY);
+	if (!mattk)
+	    impossible("bad breath attack?");	/* mouthwash needed... */
+	else
 	    buzz((int) (20 + mattk->adtyp-1), (int)mattk->damn,
 		u.ux, u.uy, u.dx, u.dy);
-	}
 	return(1);
 }
 
@@ -812,11 +844,11 @@ dospinweb()
 		case SPIKED_PIT: You("spin a web, covering up the pit.");
 			deltrap(ttmp);
 			bury_objs(u.ux, u.uy);
-			if (Invisible) newsym(u.ux, u.uy);
+			newsym(u.ux, u.uy);
 			return(1);
 		case SQKY_BOARD: pline_The("squeaky board is muffled.");
 			deltrap(ttmp);
-			if (Invisible) newsym(u.ux, u.uy);
+			newsym(u.ux, u.uy);
 			return(1);
 		case TELEP_TRAP:
 		case LEVEL_TELEP:
@@ -830,12 +862,12 @@ dospinweb()
 			You("web over the %s.",
 			    (ttmp->ttyp == TRAPDOOR) ? "trap door" : "hole");
 			deltrap(ttmp);
-			if (Invisible) newsym(u.ux, u.uy);
+			newsym(u.ux, u.uy);
 			return 1;
 		case ROLLING_BOULDER_TRAP:
 			You("spin a web, jamming the trigger.");
 			deltrap(ttmp);
-			if (Invisible) newsym(u.ux, u.uy);
+			newsym(u.ux, u.uy);
 			return(1);
 		case ARROW_TRAP:
 		case DART_TRAP:
@@ -854,19 +886,27 @@ dospinweb()
 		default:
 			impossible("Webbing over trap type %d?", ttmp->ttyp);
 			return(0);
+		}
+	else if (On_stairs(u.ux, u.uy)) {
+	    /* cop out: don't let them hide the stairs */
+	    Your("web fails to impede access to the %s.",
+		 (levl[u.ux][u.uy].typ == STAIRS) ? "stairs" : "ladder");
+	    return(1);
+		 
 	}
 	ttmp = maketrap(u.ux, u.uy, WEB);
 	if (ttmp) {
 		ttmp->tseen = 1;
 		ttmp->madeby_u = 1;
 	}
-	if (Invisible) newsym(u.ux, u.uy);
+	newsym(u.ux, u.uy);
 	return(1);
 }
 
 int
 dosummon()
 {
+	int placeholder;
 	if (u.uen < 10) {
 	    You("lack the energy to send forth a call for help!");
 	    return(0);
@@ -876,7 +916,7 @@ dosummon()
 
 	You("call upon your brethren for help!");
 	exercise(A_WIS, TRUE);
-	if (!were_summon(youmonst.data,TRUE))
+	if (!were_summon(youmonst.data, TRUE, &placeholder, (char *)0))
 		pline("But none arrive.");
 	return(1);
 }
@@ -998,8 +1038,7 @@ dogaze()
 			/* as if gazing at a sleeping anything is fruitful... */
 			You("turn to stone...");
 			killer_format = KILLED_BY;
-			killer =
-			 "deliberately gazing at Medusa's hideous countenance";
+			killer = "deliberately meeting Medusa's gaze";
 			done(STONING);
 		    }
 		}
@@ -1109,6 +1148,10 @@ int part;
 		"rear claw", "foreclaw", "clawed", "head", "rear limb",
 		"light headed", "neck", "spine", "rear claw tip",
 		"fur", "blood", "lung", "nose", "stomach" },
+	*bird_parts[] = { "wing", "eye", "face", "wing", "wing tip",
+		"foot", "wing", "winged", "head", "leg",
+		"light headed", "neck", "spine", "toe",
+		"feathers", "blood", "lung", "bill", "stomach" },
 	*horse_parts[] = { "foreleg", "eye", "face", "forehoof", "hoof tip",
 		"rear hoof", "foreclaw", "hooved", "head", "rear leg",
 		"light headed", "neck", "backbone", "rear hoof tip",
@@ -1169,9 +1212,17 @@ int part;
 		(part == ARM || part == FINGER || part == FINGERTIP ||
 		    part == HAND || part == HANDED))
 	    return humanoid_parts[part];
+	if (mptr == &mons[PM_RAVEN])
+	    return bird_parts[part];
 	if (mptr->mlet == S_CENTAUR || mptr->mlet == S_UNICORN ||
 		(mptr == &mons[PM_ROTHE] && part != HAIR))
 	    return horse_parts[part];
+	if (mptr->mlet == S_LIGHT) {
+		if (part == HANDED) return "rayed";
+		else if (part == ARM || part == FINGER ||
+				part == FINGERTIP || part == HAND) return "ray";
+		else return "beam";
+	}
 	if (mptr->mlet == S_EEL && mptr != &mons[PM_JELLYFISH])
 	    return fish_parts[part];
 	if (slithy(mptr) || (mptr->mlet == S_DRAGON && part == HAIR))
@@ -1225,7 +1276,7 @@ int damtype, dam;
 	if (u.umonnum != PM_FLESH_GOLEM && u.umonnum != PM_IRON_GOLEM)
 		return;
 	switch (damtype) {
-		case AD_ELEC: if (u.umonnum == PM_IRON_GOLEM)
+		case AD_ELEC: if (u.umonnum == PM_FLESH_GOLEM)
 				heal = dam / 6; /* Approx 1 per die */
 			break;
 		case AD_FIRE: if (u.umonnum == PM_IRON_GOLEM)
