@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)role.c	3.4	2000/08/20	*/
+/*	SCCS Id: @(#)role.c	3.4	2003/01/08	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985-1999. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -1100,7 +1100,7 @@ int buflen, rolenum, racenum, gendnum, alignnum;
 {
 	int k, gendercount = 0, aligncount = 0;
 	char buf[BUFSZ];
-	char *err_ret = " character's";
+	static char err_ret[] = " character's";
 	boolean donefirst = FALSE;
 
 	if (!suppliedbuf || buflen < 1) return err_ret;
@@ -1368,13 +1368,15 @@ role_init()
 	if (!validrace(flags.initrole, flags.initrace))
 	    flags.initrace = randrace(flags.initrole);
 
-	/* Check for a valid gender.  Try flags.igend first. */
+	/* Check for a valid gender.  If new game, check both initgend
+	 * and female.  On restore, assume flags.female is correct. */
+	if (flags.pantheon == -1) {	/* new game */
+	    if (!validgend(flags.initrole, flags.initrace, flags.female))
+		flags.female = !flags.female;
+	}
 	if (!validgend(flags.initrole, flags.initrace, flags.initgend))
-	    /* Use flags.female second.  Note that there is no way
-	     * to check for an unspecified gender.
-	     */
+	    /* Note that there is no way to check for an unspecified gender. */
 	    flags.initgend = flags.female;
-	/* Don't change flags.female; this may be a restore */
 
 	/* Check for a valid alignment */
 	if (!validalign(flags.initrole, flags.initrace, flags.initalign))
