@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)teleport.c	3.4	2002/03/09	*/
+/*	SCCS Id: @(#)teleport.c	3.4	2003/08/11	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -18,11 +18,13 @@ STATIC_DCL void FDECL(mvault_tele, (struct monst *));
  * call it to generate new monster positions with fake monster structures.
  */
 boolean
-goodpos(x, y, mtmp)
+goodpos(x, y, mtmp, gpflags)
 int x,y;
 struct monst *mtmp;
+unsigned gpflags;
 {
 	struct permonst *mdat = NULL;
+	boolean ignorewater = ((gpflags & MM_IGNOREWATER) != 0);
 
 	if (!isok(x, y)) return FALSE;
 
@@ -42,17 +44,27 @@ struct monst *mtmp;
 	if (mtmp) {
 	    struct monst *mtmp2 = m_at(x,y);
 
-	    if (mtmp2 && mtmp2 != mtmp)
+	    /* Be careful with long worms.  A monster may be placed back in
+	     * its own location.  Normally, if m_at() returns the same monster
+	     * that we're trying to place, the monster is being placed in its
+	     * own location.  However, that is not correct for worm segments,
+	     * because all the segments of the worm return the same m_at().
+	     * Actually we overdo the check a little bit--a worm can't be placed
+	     * in its own location, period.  If we just checked for mtmp->mx
+	     * != x || mtmp->my != y, we'd miss the case where we're called
+	     * to place the worm segment and the worm's head is at x,y.
+	     */
+	    if (mtmp2 && (mtmp2 != mtmp || mtmp->wormno))
 		return FALSE;
 
 	    mdat = mtmp->data;
-	    if (is_pool(x,y)) {
+	    if (is_pool(x,y) && !ignorewater) {
 		if (mtmp == &youmonst)
 			return !!(HLevitation || Flying || Wwalking ||
 					Swimming || Amphibious);
 		else	return (is_flyer(mdat) || is_swimmer(mdat) ||
 							is_clinger(mdat));
-	    } else if (mdat->mlet == S_EEL && rn2(13)) {
+	    } else if (mdat->mlet == S_EEL && rn2(13) && !ignorewater) {
 		return FALSE;
 	    } else if (is_lava(x,y)) {
 		if (mtmp == &youmonst)
@@ -62,7 +74,10 @@ struct monst *mtmp;
 	    }
 	    if (passes_walls(mdat) && may_passwall(x,y)) return TRUE;
 	}
-	if (!ACCESSIBLE(levl[x][y].typ)) return FALSE;
+	if (!ACCESSIBLE(levl[x][y].typ)) {
+		if (!(is_pool(x,y) && ignorewater)) return FALSE;
+	}
+
 	if (closed_door(x, y) && (!mdat || !amorphous(mdat)))
 		return FALSE;
 	if (sobj_at(BOULDER, x, y) && (!mdat || !throws_rocks(mdat)))
@@ -83,6 +98,16 @@ enexto(cc, xx, yy, mdat)
 coord *cc;
 register xchar xx, yy;
 struct permonst *mdat;
+{
+	return enexto_core(cc, xx, yy, mdat, 0);
+}
+
+boolean
+enexto_core(cc, xx, yy, mdat, entflags)
+coord *cc;
+register xchar xx, yy;
+struct permonst *mdat;
+unsigned entflags;
 {
 #define MAX_GOOD 15
     coord good[MAX_GOOD], *good_ptr;
@@ -111,28 +136,28 @@ struct permonst *mdat;
 	ymax = min(ROWNO-1, yy+range);
 
 	for (x = xmin; x <= xmax; x++)
-	    if (goodpos(x, ymin, &fakemon)) {
+	    if (goodpos(x, ymin, &fakemon, entflags)) {
 		good_ptr->x = x;
 		good_ptr->y = ymin ;
 		/* beware of accessing beyond segment boundaries.. */
 		if (good_ptr++ == &good[MAX_GOOD-1]) goto full;
 	    }
 	for (x = xmin; x <= xmax; x++)
-	    if (goodpos(x, ymax, &fakemon)) {
+	    if (goodpos(x, ymax, &fakemon, entflags)) {
 		good_ptr->x = x;
 		good_ptr->y = ymax ;
 		/* beware of accessing beyond segment boundaries.. */
 		if (good_ptr++ == &good[MAX_GOOD-1]) goto full;
 	    }
 	for (y = ymin+1; y < ymax; y++)
-	    if (goodpos(xmin, y, &fakemon)) {
+	    if (goodpos(xmin, y, &fakemon, entflags)) {
 		good_ptr->x = xmin;
 		good_ptr-> y = y ;
 		/* beware of accessing beyond segment boundaries.. */
 		if (good_ptr++ == &good[MAX_GOOD-1]) goto full;
 	    }
 	for (y = ymin+1; y < ymax; y++)
-	    if (goodpos(xmax, y, &fakemon)) {
+	    if (goodpos(xmax, y, &fakemon, entflags)) {
 		good_ptr->x = xmax;
 		good_ptr->y = y ;
 		/* beware of accessing beyond segment boundaries.. */
@@ -195,40 +220,58 @@ register int x, y;
 boolean trapok;
 {
 	if (!trapok && t_at(x, y)) return FALSE;
-	if (!goodpos(x, y, &youmonst)) return FALSE;
+	if (!goodpos(x, y, &youmonst, 0)) return FALSE;
 	if (!tele_jump_ok(u.ux, u.uy, x, y)) return FALSE;
+	if (!in_out_region(x, y)) return FALSE;
 	return TRUE;
 }
 
 void
-teleds(nux, nuy)
+teleds(nux, nuy, allow_drag)
 register int nux,nuy;
+boolean allow_drag;
 {
-	boolean dont_teleport_ball = FALSE;
+	boolean ball_active = (Punished && uball->where != OBJ_FREE),
+		ball_still_in_range = FALSE;
 
-	if (Punished) {
-	    /* If they're teleporting to a position where the ball doesn't need
-	     * to be moved, don't place the ball.  Especially useful when this
-	     * function is being called for crawling out of water instead of
-	     * real teleportation.
-	     */
+	/* If they have to move the ball, then drag if allow_drag is true;
+	 * otherwise they are teleporting, so unplacebc().  
+	 * If they don't have to move the ball, then always "drag" whether or
+	 * not allow_drag is true, because we are calling that function, not
+	 * to drag, but to move the chain.  *However* there are some dumb
+	 * special cases:
+	 *    0				 0
+	 *   _X  move east       ----->  X_
+	 *    @				  @
+	 * These are permissible if teleporting, but not if dragging.  As a
+	 * result, drag_ball() needs to know about allow_drag and might end
+	 * up dragging the ball anyway.  Also, drag_ball() might find that
+	 * dragging the ball is completely impossible (ball in range but there's
+	 * rock in the way), in which case it teleports the ball on its own.
+	 */
+	if (ball_active) {
 	    if (!carried(uball) && distmin(nux, nuy, uball->ox, uball->oy) <= 2)
-		dont_teleport_ball = TRUE;
-	    else
-		unplacebc();
+		ball_still_in_range = TRUE; /* don't have to move the ball */
+	    else {
+		/* have to move the ball */
+		if (!allow_drag || distmin(u.ux, u.uy, nux, nuy) > 1) {
+		    /* we should not have dist > 1 and allow_drag at the same
+		     * time, but just in case, we must then revert to teleport.
+		     */
+		    allow_drag = FALSE;
+		    unplacebc();
+		}
+	    }
 	}
 	u.utrap = 0;
 	u.ustuck = 0;
 	u.ux0 = u.ux;
 	u.uy0 = u.uy;
-	u.ux = nux;
-	u.uy = nuy;
-	fill_pit(u.ux0, u.uy0); /* do this now so that cansee() is correct */
 
 	if (hides_under(youmonst.data))
 		u.uundetected = OBJ_AT(nux, nuy);
 	else if (youmonst.data->mlet == S_EEL)
-		u.uundetected = is_pool(u.ux, u.uy);
+		u.uundetected = is_pool(nux, nuy);
 	else {
 		u.uundetected = 0;
 		/* mimics stop being unnoticed */
@@ -238,21 +281,32 @@ register int nux,nuy;
 
 	if (u.uswallow) {
 		u.uswldtim = u.uswallow = 0;
+		if (Punished && !ball_active) {
+		    /* ensure ball placement, like unstuck */
+		    ball_active = TRUE;
+		    allow_drag = FALSE;
+		}
 		docrt();
 	}
-	if (Punished) {
-	    if (dont_teleport_ball) {
+	if (ball_active) {
+	    if (ball_still_in_range || allow_drag) {
 		int bc_control;
 		xchar ballx, bally, chainx, chainy;
 		boolean cause_delay;
 
-		/* this should only drag the chain (and never give a near-
-		   capacity message) since we already checked ball distance */
-		(void) drag_ball(u.ux, u.uy, &bc_control, &ballx, &bally,
-					&chainx, &chainy, &cause_delay);
-		move_bc(0, bc_control, ballx, bally, chainx, chainy);
-	    } else
-		 placebc();
+		if (drag_ball(nux, nuy, &bc_control, &ballx, &bally,
+				    &chainx, &chainy, &cause_delay, allow_drag))
+		    move_bc(0, bc_control, ballx, bally, chainx, chainy);
+	    }
+	}
+	/* must set u.ux, u.uy after drag_ball(), which may need to know
+	   the old position if allow_drag is true... */
+	u.ux = nux;
+	u.uy = nuy;
+	fill_pit(u.ux0, u.uy0);
+	if (ball_active) {
+	    if (!ball_still_in_range && !allow_drag)
+		placebc();
 	}
 	initrack(); /* teleports mess up tracking monsters without this */
 	update_player_regions();
@@ -279,7 +333,8 @@ register int nux,nuy;
 }
 
 boolean
-safe_teleds()
+safe_teleds(allow_drag)
+boolean allow_drag;
 {
 	register int nux, nuy, tcnt = 0;
 
@@ -289,7 +344,7 @@ safe_teleds()
 	} while (!teleok(nux, nuy, (boolean)(tcnt > 200)) && ++tcnt <= 400);
 
 	if (tcnt <= 400) {
-		teleds(nux, nuy);
+		teleds(nux, nuy, allow_drag);
 		return TRUE;
 	} else
 		return FALSE;
@@ -302,7 +357,7 @@ vault_tele()
 	coord c;
 
 	if (croom && somexy(croom, &c) && teleok(c.x,c.y,FALSE)) {
-		teleds(c.x,c.y);
+		teleds(c.x,c.y,FALSE);
 		return;
 	}
 	tele();
@@ -363,7 +418,7 @@ tele()
 	    You_feel("disoriented for a moment.");
 	    return;
 	}
-	if (Teleport_control
+	if ((Teleport_control && !Stunned)
 #ifdef WIZARD
 			    || wizard
 #endif
@@ -387,14 +442,14 @@ tele()
 		    /* possible extensions: introduce a small error if
 		       magic power is low; allow transfer to solid rock */
 		    if (teleok(cc.x, cc.y, FALSE)) {
-			teleds(cc.x, cc.y);
+			teleds(cc.x, cc.y, FALSE);
 			return;
 		    }
 		    pline("Sorry...");
 		}
 	}
 
-	(void) safe_teleds();
+	(void) safe_teleds(FALSE);
 }
 
 int
@@ -512,6 +567,7 @@ level_tele()
 	d_level newlevel;
 	const char *escape_by_flying = 0;	/* when surviving dest of -N */
 	char buf[BUFSZ];
+	boolean force_dest = FALSE;
 
 	if ((u.uhave.amulet || In_endgame(&u.uz) || In_sokoban(&u.uz))
 #ifdef WIZARD
@@ -521,7 +577,7 @@ level_tele()
 	    You_feel("very disoriented for a moment.");
 	    return;
 	}
-	if (Teleport_control
+	if ((Teleport_control && !Stunned)
 #ifdef WIZARD
 	   || wizard
 #endif
@@ -531,12 +587,53 @@ level_tele()
 
 	    Strcpy(qbuf, "To what level do you want to teleport?");
 	    do {
-		if (++trycnt == 2) Strcat(qbuf, " [type a number]");
+		if (++trycnt == 2) {
+#ifdef WIZARD
+			if (wizard) Strcat(qbuf, " [type a number or ? for a menu]");
+			else
+#endif
+			Strcat(qbuf, " [type a number]");
+		}
 		getlin(qbuf, buf);
-		if (!strcmp(buf,"\033"))	/* cancelled */
+		if (!strcmp(buf,"\033")) {	/* cancelled */
+		    if (Confusion && rnl(5)) {
+			pline("Oops...");
+			goto random_levtport;
+		    }
 		    return;
-		else if (!strcmp(buf,"*"))
+		} else if (!strcmp(buf,"*")) {
 		    goto random_levtport;
+		} else if (Confusion && rnl(5)) {
+		    pline("Oops...");
+		    goto random_levtport;
+		}
+#ifdef WIZARD
+		if (wizard && !strcmp(buf,"?")) {
+		    schar destlev = 0;
+		    xchar destdnum = 0;
+
+		    if ((newlev = (int)print_dungeon(TRUE, &destlev, &destdnum))) {
+			newlevel.dnum = destdnum;
+			newlevel.dlevel = destlev;
+			if (In_endgame(&newlevel) && !In_endgame(&u.uz)) {
+				Sprintf(buf,
+				    "Destination is earth level");
+				if (!u.uhave.amulet) {
+					struct obj *obj;
+					obj = mksobj(AMULET_OF_YENDOR,
+							TRUE, FALSE);
+					if (obj) {
+						obj = addinv(obj);
+						Strcat(buf, " with the amulet");
+					}
+				}
+				assign_level(&newlevel, &earth_level);
+				pline("%s.", buf);
+			}
+			force_dest = TRUE;
+		    } else return;
+		} else
+#endif
 		if ((newlev = lev_by_name(buf)) == 0) newlev = atoi(buf);
 	    } while (!newlev && !digit(buf[0]) &&
 		     (buf[0] != '-' || !digit(buf[1])) &&
@@ -551,9 +648,14 @@ level_tele()
 		    is_silent(youmonst.data) ? "writhe" : "scream");
 		display_nhwindow(WIN_MESSAGE, FALSE);
 		You("cease to exist.");
+		if (invent) Your("possessions land on the %s with a thud.",
+				surface(u.ux, u.uy));
 		killer_format = NO_KILLER_PREFIX;
 		killer = "committed suicide";
 		done(DIED);
+		pline("An energized cloud of dust begins to coalesce.");
+		Your("body rematerializes%s.", invent ?
+			", and you gather up all your possessions" : "");
 		return;
 	    }
 
@@ -604,7 +706,15 @@ level_tele()
 
 	killer = 0;		/* still alive, so far... */
 
-	if (newlev < 0) {
+	if (newlev < 0 && !force_dest) {
+		if (*u.ushops0) {
+		    /* take unpaid inventory items off of shop bills */
+		    in_mklev = TRUE;	/* suppress map update */
+		    u_left_shop(u.ushops0, TRUE);
+		    /* you're now effectively out of the shop */
+		    *u.ushops0 = *u.ushops = '\0';
+		    in_mklev = FALSE;
+		}
 		if (newlev <= -10) {
 			You("arrive in heaven.");
 			verbalize("Thou art early, but we'll admit thee.");
@@ -627,8 +737,8 @@ level_tele()
 		    pline("Unfortunately, you don't know how to fly.");
 		    You("plummet a few thousand feet to your death.");
 		    Sprintf(buf,
-				"teleported out of the dungeon and fell to %s death",
-				uhis());
+			  "teleported out of the dungeon and fell to %s death",
+			    uhis());
 		    killer = buf;
 		    killer_format = NO_KILLER_PREFIX;
 		}
@@ -659,6 +769,9 @@ level_tele()
 	} else if (u.uz.dnum == medusa_level.dnum &&
 	    newlev >= dungeons[u.uz.dnum].depth_start +
 						dunlevs_in_dungeon(&u.uz)) {
+#ifdef WIZARD
+	    if (!(wizard && force_dest))
+#endif
 	    find_hell(&newlevel);
 	} else {
 	    /* if invocation did not yet occur, teleporting into
@@ -681,6 +794,9 @@ level_tele()
 	     * we must translate newlev to a number relative to the
 	     * current dungeon.
 	     */
+#ifdef WIZARD
+	    if (!(wizard && force_dest))
+#endif
 	    get_level(&newlevel, newlev);
 	}
 	schedule_goto(&newlevel, FALSE, FALSE, 0, (char *)0, (char *)0);
@@ -770,7 +886,7 @@ struct monst *mtmp;
 {
 	register int xx, yy;
 
-	if (!goodpos(x, y, mtmp)) return FALSE;
+	if (!goodpos(x, y, mtmp, 0)) return FALSE;
 	/*
 	 * Check for restricted areas present in some special levels.
 	 *
@@ -859,16 +975,18 @@ register int x, y;
 }
 
 /* place a monster at a random location, typically due to teleport */
-void
-rloc(mtmp)
+/* return TRUE if successful, FALSE if not */
+boolean
+rloc(mtmp, suppress_impossible)
 struct monst *mtmp;	/* mx==0 implies migrating monster arrival */
+boolean suppress_impossible;
 {
 	register int x, y, trycount;
 
 #ifdef STEED
 	if (mtmp == u.usteed) {
 	    tele();
-	    return;
+	    return TRUE;
 	}
 #endif
 
@@ -882,7 +1000,7 @@ struct monst *mtmp;	/* mx==0 implies migrating monster arrival */
 	    /* if the wiz teleports away to heal, try the up staircase,
 	       to block the player's escaping before he's healed
 	       (deliberately use `goodpos' rather than `rloc_pos_ok' here) */
-	    if (goodpos(x, y, mtmp))
+	    if (goodpos(x, y, mtmp, 0))
 		goto found_xy;
 	}
 
@@ -891,22 +1009,24 @@ struct monst *mtmp;	/* mx==0 implies migrating monster arrival */
 	    x = rn1(COLNO-3,2);
 	    y = rn2(ROWNO);
 	    if ((trycount < 500) ? rloc_pos_ok(x, y, mtmp)
-				 : goodpos(x, y, mtmp))
+				 : goodpos(x, y, mtmp, 0))
 		goto found_xy;
 	} while (++trycount < 1000);
 
 	/* last ditch attempt to find a good place */
 	for (x = 2; x < COLNO - 1; x++)
 	    for (y = 0; y < ROWNO; y++)
-		if (goodpos(x, y, mtmp))
+		if (goodpos(x, y, mtmp, 0))
 		    goto found_xy;
 
 	/* level either full of monsters or somehow faulty */
-	impossible("rloc(): couldn't relocate monster");
-	return;
+	if (!suppress_impossible)
+		impossible("rloc(): couldn't relocate monster");
+	return FALSE;
 
  found_xy:
 	rloc_to(mtmp, x, y);
+	return TRUE;
 }
 
 STATIC_OVL void
@@ -917,11 +1037,11 @@ struct monst *mtmp;
 	coord c;
 
 	if (croom && somexy(croom, &c) &&
-				goodpos(c.x, c.y, mtmp)) {
+				goodpos(c.x, c.y, mtmp, 0)) {
 		rloc_to(mtmp, c.x, c.y);
 		return;
 	}
-	rloc(mtmp);
+	(void) rloc(mtmp, FALSE);
 }
 
 boolean
@@ -955,7 +1075,7 @@ int in_sight;
 	     * the guard isn't going to come for it...
 	     */
 	    if (trap->once) mvault_tele(mtmp);
-	    else rloc(mtmp);
+	    else (void) rloc(mtmp, FALSE);
 
 	    if (in_sight) {
 		if (canseemon(mtmp))
@@ -1058,7 +1178,7 @@ register struct obj *obj;
 	    tx = rn1(COLNO-3,2);
 	    ty = rn2(ROWNO);
 	    if (!--try_limit) break;
-	} while (!goodpos(tx, ty, (struct monst *)0) ||
+	} while (!goodpos(tx, ty, (struct monst *)0, 0) ||
 		/* bug: this lacks provision for handling the Wizard's tower */
 		 (restricted_fall &&
 		  (!within_bounded_area(tx, ty, dndest.lx, dndest.ly,
@@ -1090,15 +1210,11 @@ register struct obj *obj;
 int
 random_teleport_level()
 {
-	int nlev, max_depth, min_depth;
+	int nlev, max_depth, min_depth,
+	    cur_depth = (int)depth(&u.uz);
 
 	if (!rn2(5) || Is_knox(&u.uz))
-		return (int)depth(&u.uz);
-
-	/* Get a random value relative to the current dungeon */
-	/* Range is 1 to current+3, current not counting */
-	nlev = rnd((int)depth(&u.uz) + 2);
-	if (nlev >= (int)depth(&u.uz)) nlev++;
+	    return cur_depth;
 
 	/* What I really want to do is as follows:
 	 * -- If in a dungeon that goes down, the new level is to be restricted
@@ -1115,10 +1231,20 @@ random_teleport_level()
 	 * above; endgame is handled in the caller due to its different
 	 * message ("disoriented").
 	 * --KAA
+	 * 3.4.2: explicitly handle quest here too, to fix the problem of
+	 * monsters sometimes level teleporting out of it into main dungeon.
+	 * Also prevent monsters reaching the Sanctum prior to invocation.
 	 */
-	min_depth = 1;
+	min_depth = In_quest(&u.uz) ? dungeons[u.uz.dnum].depth_start : 1;
 	max_depth = dunlevs_in_dungeon(&u.uz) +
 			(dungeons[u.uz.dnum].depth_start - 1);
+	/* can't reach the Sanctum if the invocation hasn't been performed */
+	if (Inhell && !u.uevent.invoked) max_depth -= 1;
+
+	/* Get a random value relative to the current dungeon */
+	/* Range is 1 to current+3, current not counting */
+	nlev = rn2(cur_depth + 3 - min_depth) + min_depth;
+	if (nlev >= cur_depth) nlev++;
 
 	if (nlev > max_depth) {
 	    nlev = max_depth;
@@ -1127,9 +1253,9 @@ random_teleport_level()
 	}
 	if (nlev < min_depth) {
 	    nlev = min_depth;
-	    if ((int)depth(&u.uz) == min_depth) {
-		nlev += rnd(3);
-		if (nlev > max_depth)
+	    if (nlev == cur_depth) {
+	        nlev += rnd(3);
+	        if (nlev > max_depth)
 		    nlev = max_depth;
 	    }
 	}
@@ -1153,12 +1279,12 @@ boolean give_feedback;
 	    if (give_feedback)
 		You("are no longer inside %s!", mon_nam(mtmp));
 	    unstuck(mtmp);
-	    rloc(mtmp);
+	    (void) rloc(mtmp, FALSE);
 	} else if (is_rider(mtmp->data) && rn2(13) &&
 		   enexto(&cc, u.ux, u.uy, mtmp->data))
 	    rloc_to(mtmp, cc.x, cc.y);
 	else
-	    rloc(mtmp);
+	    (void) rloc(mtmp, FALSE);
 	return TRUE;
 }
 
