@@ -1,4 +1,4 @@
-/*    SCCS Id: @(#)amidos.c     3.2    96/02/16
+/*    SCCS Id: @(#)amidos.c     3.2    2000/01/12
 /* Copyright (c) Olaf Seibert, Nijmegen, The Netherlands, 1988,1990.    */
 /* Copyright (c) Kenneth Lorber, Bethesda, Maryland, 1991,1992,1993,1996.  */
 /* NetHack may be freely redistributed.  See license for details.	*/
@@ -18,7 +18,7 @@
 #include <intuition/intuition.h>
 
 #undef COUNT
-#ifdef __SASC_60
+#if defined(__SASC_60) || defined(__GNUC__)
 # include <proto/exec.h>
 # include <proto/dos.h>
 #endif
@@ -41,7 +41,7 @@ int Enable_Abort = 0;   /* for stdio package */
 #endif
 
 /* Initial path, so we can find NetHack.cnf */
-char PATH[PATHLEN] = "Ram:;df0:;NetHack:";
+char PATH[PATHLEN] = "NetHack:";
 
 static boolean record_exists(void);
 
@@ -80,14 +80,15 @@ int
 dosh()
 {
     int i;
-    char buf[ 500 ];
+    char buf[ BUFSZ ];
     extern struct ExecBase *SysBase;
 
     /* Only under 2.0 and later ROMs do we have System() */
     if( SysBase->LibNode.lib_Version >= 37 && !amibbs)
     {
 	getlin("Enter CLI Command...", buf );
-	i = System( buf, NULL );
+	if (buf[0] != '\033')
+	    i = System( buf, NULL );
     }
     else
     {
@@ -225,6 +226,7 @@ const char *path, *files;
 
 /* This size makes that most files can be copied with two Read()/Write()s */
 
+#if 0 /* Unused */
 #define COPYSIZE    4096
 
 char *CopyFile(from, to)
@@ -257,7 +259,7 @@ const char *from, *to;
     free(buffer);
     return error;
 }
-
+#endif
 
 /* this should be replaced */
 saveDiskPrompt(start)
@@ -380,6 +382,7 @@ register const char *name, *mode;
 
     /* Try the default directory first.  Then look along PATH.
      */
+    if (strlen(name) >= BUFSIZ) return( NULL );
     strcpy(buf, name);
     if (theLock = Lock(buf, SHARED_LOCK)) {
 	UnLock(theLock);
@@ -395,6 +398,7 @@ register const char *name, *mode;
 	}
 	if (lastch != ':' && lastch != '/' && bp != buf)
 	    *bp++ = '/';
+	if (bp + strlen(name) > buf + BUFSIZ - 1) return( NULL );
 	strcpy(bp, name);
 	if (theLock = Lock(buf, SHARED_LOCK)) {
 	    UnLock(theLock);
@@ -421,7 +425,7 @@ register const char *name, *mode;
 static BPTR OrgDirLock = NO_LOCK;
 
 chdir(dir)
-const char *dir;
+char *dir;
 {
     extern char orgdir[];
 
