@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)macmenu.c	3.3	99/11/24	*/
+/*	SCCS Id: @(#)macmenu.c	3.4	1999/11/24	*/
 /*      Copyright (c) Macintosh NetHack Port Team, 1993.          */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -23,17 +23,22 @@
 /******** Application Defines ********/
 #include "hack.h"
 #include "mactty.h"
-#include "macpopup.h"
 #include "macwin.h"
+#include "macpopup.h"
 #include "patchlevel.h"
 
 /******** Toolbox Defines ********/
+#if !TARGET_API_MAC_CARBON
 #include <Menus.h>
 #include <Devices.h>
 #include <Resources.h>
+#include <TextUtils.h>
 #include <ToolUtils.h>
 #include <Sound.h>
-#include <LowMem.h>		// for LMGetWindowList
+#endif
+
+/* Borrowed from the Mac tty port */
+extern WindowPtr _mt_window;
 
 /******** Local Defines ********/
 
@@ -81,11 +86,9 @@ enum
 	____Apple__1,
 
 	/* File */
-	menuFileOpenMap = 1,
-	menuFileRedraw,
+	menuFileRedraw = 1,
 	menuFilePrevMsg,
 	menuFileCleanup,
-	menuFileClose,
 	____File___1,
 	menuFilePlayMode,
 	menuFileEnterExplore,
@@ -207,14 +210,12 @@ static	short		kAdjustWizardMenu = 1;
 
 
 /******** Prototypes ********/
+#if !TARGET_API_MAC_CARBON
 static	void alignAD(Rect *, short);
+#endif
 static	void mustGetMenuAlerts(void);
 static	void menuError(short);
-static	pascal void drawANUserItem(WindowPtr, short);
 static	void aboutNetHack(void);
-static	void openMap(void);
-static	void closeFrontWindow(void);
-static	void optionEditor(void);
 static	void askSave(void);
 static	void askQuit(void);
 
@@ -243,9 +244,9 @@ static	void askQuit(void);
 #define CH_ESCAPE	0x001b
 
 static void ask_restring(const char *cstr, unsigned char *pstr);
-static void ask_enable(WindowPtr wind, short item, int enable);
-static pascal void ask_redraw(WindowPtr wind, DialogItemIndex item);
-static pascal Boolean ask_filter(WindowPtr wind, EventRecord *event, DialogItemIndex *item);
+static void ask_enable(DialogRef wind, short item, int enable);
+static pascal void ask_redraw(DialogRef wind, DialogItemIndex item);
+static pascal Boolean ask_filter(DialogRef wind, EventRecord *event, DialogItemIndex *item);
 #define noresource(t,n)	{SysBeep(3); ExitToShell();}
 #define fatal(s)	{SysBeep(3); ExitToShell();}
 
@@ -259,7 +260,7 @@ static int askselect[RSRC_ASK_MAX];
 
 static RGBColor
 	blackcolor = {0x0000, 0x0000, 0x0000},
-	indentcolor = {0x4000, 0x4000, 0x4000},
+//	indentcolor = {0x4000, 0x4000, 0x4000},
 	darkcolor = {0x8000, 0x8000, 0x8000},
 	backcolor = {0xdddd, 0xdddd, 0xdddd},
 	lightcolor = {0xffff, 0xffff, 0xffff},
@@ -284,7 +285,7 @@ ask_restring (const char *cstr, unsigned char *pstr)
 
 /* Enable the dialog item with the given index */
 static void
-ask_enable (WindowPtr wind, short item, int enable)
+ask_enable (DialogRef wind, short item, int enable)
 {
 	short type;
 	Handle handle;
@@ -292,26 +293,26 @@ ask_enable (WindowPtr wind, short item, int enable)
 
 
 	/* Enable or disable the appropriate item */
-	GetDItem(wind, item, &type, &handle, &rect);
+	GetDialogItem(wind, item, &type, &handle, &rect);
 	if (enable)	type &= ~itemDisable;
 	else		type |= itemDisable;
 	HiliteControl((ControlHandle)handle, enable ? 0 : 255);
-	SetDItem(wind, item, type, handle, &rect);
+	SetDialogItem(wind, item, type, handle, &rect);
 	return;
 }
 
 
 static pascal void
-ask_redraw (WindowPtr wind, DialogItemIndex item)
+ask_redraw (DialogRef wind, DialogItemIndex item)
 {
 	short type;
 	Handle handle;
 	Rect rect;
-	static char	*modechar = " XD";
+	static char	*modechar = "NED";
 
 
 	/* Which item shall we redraw? */
-	GetDItem(wind, item, &type, &handle, &rect);
+	GetDialogItem(wind, item, &type, &handle, &rect);
 	switch (item) {
 		case RSRC_ASK_DEFAULT:
 			PenSize(3, 3);
@@ -435,7 +436,7 @@ ask_redraw (WindowPtr wind, DialogItemIndex item)
 
 
 static pascal Boolean
-ask_filter (WindowPtr wind, EventRecord *event, DialogItemIndex *item)
+ask_filter (DialogRef wind, EventRecord *event, DialogItemIndex *item)
 {
 	short ch, key;
 
@@ -448,13 +449,32 @@ ask_filter (WindowPtr wind, EventRecord *event, DialogItemIndex *item)
 			/* Handle equivalents for OK */
 			if ((ch == CH_RETURN) || (key == KEY_RETURN) ||
 				(ch == CH_ENTER) || (key == KEY_ENTER)) {
-				if ((*((DialogRecord *)wind)->textH)->teLength) {
+				if (GetDialogTextEditHandle(wind)[0]->teLength) {
 					FlashButton(wind, RSRC_ASK_PLAY);
 					*item = RSRC_ASK_PLAY;
 				} else
 					*item = 0;
 				return (TRUE);
 			}
+			/* Handle equivalents for Normal/Explore/Debug */
+			if ((event->modifiers & cmdKey) && (ch == 'n')) {
+				currmode = 0;
+				ask_redraw(wind, RSRC_ASK_MODE);
+				*item = RSRC_ASK_MODE;
+				return (TRUE);
+			}
+			if ((event->modifiers & cmdKey) && (ch == 'e')) {
+				currmode = 1;
+				ask_redraw(wind, RSRC_ASK_MODE);
+				*item = RSRC_ASK_MODE;
+				return (TRUE);
+			}
+			if ((event->modifiers & cmdKey) && (ch == 'd')) {
+				currmode = 2;
+				ask_redraw(wind, RSRC_ASK_MODE);
+				*item = RSRC_ASK_MODE;
+				return (TRUE);
+			}
 			/* Handle equivalents for Cancel and Quit */
 			if ((ch == CH_ESCAPE) || (key == KEY_ESCAPE) ||
 				((event->modifiers & cmdKey) && (ch == 'q')) ||
@@ -476,27 +496,27 @@ ask_filter (WindowPtr wind, EventRecord *event, DialogItemIndex *item)
 void mac_askname ()
 {
 	GrafPtr oldport;
-	WindowPtr askdialog;
+	DialogRef askdialog;
 	short i, j, item, type;
 	Handle handle;
 	Rect rect;
 	Str255 str;
 	Point pt;
-	UserItemUPP redraw = NewUserItemProc(ask_redraw);
-	ModalFilterUPP filter = NewModalFilterProc(ask_filter);
+	UserItemUPP redraw = NewUserItemUPP(ask_redraw);
+	ModalFilterUPP filter = NewModalFilterUPP(ask_filter);
 
 
 	/* Create the dialog */
-	if (!(askdialog = GetNewDialog(RSRC_ASK, NULL, (WindowPtr)-1)))
+	if (!(askdialog = GetNewDialog(RSRC_ASK, NULL, (WindowRef)-1)))
 	    noresource('DLOG', RSRC_ASK);
 	GetPort(&oldport);
-	SetPort(askdialog);
+	SetPortDialogPort(askdialog);
 
 	/* Initialize the name text item */
 	ask_restring(plname, str);
 	if (plname[0]) {
-	    GetDItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
-	    SetIText(handle, str);
+	    GetDialogItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
+	    SetDialogItemText(handle, str);
 	}
 #if 0
 	{
@@ -514,13 +534,13 @@ void mac_askname ()
 				if (**h > 31) {
 					**h = 31;
 				}
-				BlockMove (*h, pName , **h + 1);
+				BlockMove (*h, pName, **h + 1);
 				DisposeHandle (h);
 			}
 		}
 		if (pName [0]) {
-			GetDItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
-			SetIText(handle, pName);
+			GetDialogItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
+			SetDialogItemText(handle, pName);
 			if (pName [0] > 2 && pName [pName [0] - 1] == '-') {
 			    short role = (*pANR).anMenu[anRole];
 			    char suffix = (char) pName[pName[0]],
@@ -535,7 +555,7 @@ void mac_askname ()
 		}
 	}
 #endif
-	SelIText(askdialog, RSRC_ASK_NAME, 0, 32767);
+	SelectDialogItemText(askdialog, RSRC_ASK_NAME, 0, 32767);
 
 	/* Initialize the role popup menu */
 	if (!(askmenu[RSRC_ASK_ROLE] = NewMenu(RSRC_ASK_ROLE, "\p")))
@@ -605,89 +625,89 @@ void mac_askname ()
 
 	/* Set the redraw procedures */
 	for (item = RSRC_ASK_DEFAULT; item <= RSRC_ASK_MODE; item++) {
-	    GetDItem(askdialog, item, &type, &handle, &rect);
-	    SetDItem(askdialog, item, type, (Handle)redraw, &rect);
+	    GetDialogItem(askdialog, item, &type, &handle, &rect);
+	    SetDialogItem(askdialog, item, type, (Handle)redraw, &rect);
 	}
 
 	/* Handle dialog events */
 	do {
 	    /* Adjust the Play button */
 	    ask_enable(askdialog, RSRC_ASK_PLAY,
-				(*((DialogRecord *)askdialog)->textH)->teLength);
+				GetDialogTextEditHandle(askdialog)[0]->teLength);
 
 	    /* Adjust the race popup menu */
 	    i = j = currrace;
 	    do {
 	    	if (validrace(currrole, j)) {
-	    		EnableItem(askmenu[RSRC_ASK_RACE], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_RACE], j+1,
+	    		EnableMenuItem(askmenu[RSRC_ASK_RACE], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_RACE], j+1,
 	    				currrace == j);
 	    	} else {
-	    		DisableItem(askmenu[RSRC_ASK_RACE], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_RACE], j+1, FALSE);
+	    		DisableMenuItem(askmenu[RSRC_ASK_RACE], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_RACE], j+1, FALSE);
 	    		if ((currrace == j) && !races[++currrace].noun)
 	    			currrace = 0;
 	    	}
 	    	if (!races[++j].noun) j = 0;
 	    } while (i != j);
 	    if (currrace != i) {
-	    	GetDItem(askdialog, RSRC_ASK_RACE, &type, &handle, &rect);
-	    	InvalRect(&rect);
+	    	GetDialogItem(askdialog, RSRC_ASK_RACE, &type, &handle, &rect);
+	    	InvalWindowRect(GetDialogWindow(askdialog), &rect);
 	    }
 
 	    /* Adjust the gender popup menu */
 	    i = j = currgend;
 	    do {
 	    	if (validgend(currrole, currrace, j)) {
-	    		EnableItem(askmenu[RSRC_ASK_GEND], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_GEND], j+1,
+	    		EnableMenuItem(askmenu[RSRC_ASK_GEND], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_GEND], j+1,
 	    				currgend == j);
 	    	} else {
-	    		DisableItem(askmenu[RSRC_ASK_GEND], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_GEND], j+1, FALSE);
+	    		DisableMenuItem(askmenu[RSRC_ASK_GEND], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_GEND], j+1, FALSE);
 	    		if ((currgend == j) && (++currgend >= ROLE_GENDERS))
 	    			currgend = 0;
 	    	}
 	    	if (++j >= ROLE_GENDERS) j = 0;
 	    } while (i != j);
 	    if (currgend != i) {
-	    	GetDItem(askdialog, RSRC_ASK_GEND, &type, &handle, &rect);
-	    	InvalRect(&rect);
+	    	GetDialogItem(askdialog, RSRC_ASK_GEND, &type, &handle, &rect);
+	    	InvalWindowRect(GetDialogWindow(askdialog), &rect);
 	    }
 
 	    /* Adjust the alignment popup menu */
 	    i = j = curralign;
 	    do {
 	    	if (validalign(currrole, currrace, j)) {
-	    		EnableItem(askmenu[RSRC_ASK_ALIGN], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_ALIGN], j+1,
+	    		EnableMenuItem(askmenu[RSRC_ASK_ALIGN], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_ALIGN], j+1,
 	    				curralign == j);
 	    	} else {
-	    		DisableItem(askmenu[RSRC_ASK_ALIGN], j+1);
-	    		CheckItem(askmenu[RSRC_ASK_ALIGN], j+1, FALSE);
+	    		DisableMenuItem(askmenu[RSRC_ASK_ALIGN], j+1);
+	    		CheckMenuItem(askmenu[RSRC_ASK_ALIGN], j+1, FALSE);
 	    		if ((curralign == j) && (++curralign >= ROLE_ALIGNS))
 	    			curralign = 0;
 	    	}
 	    	if (++j >= ROLE_ALIGNS) j = 0;
 	    } while (i != j);
 	    if (curralign != i) {
-	    	GetDItem(askdialog, RSRC_ASK_ALIGN, &type, &handle, &rect);
-	    	InvalRect(&rect);
+	    	GetDialogItem(askdialog, RSRC_ASK_ALIGN, &type, &handle, &rect);
+	    	InvalWindowRect(GetDialogWindow(askdialog), &rect);
 	    }
 
 	    /* Adjust the role popup menu */
 	    for (i = 0; roles[i].name.m; i++) {
 	    	ask_restring((currgend && roles[i].name.f) ?
 	    			roles[i].name.f : roles[i].name.m, str);
-	    	SetItem(askmenu[RSRC_ASK_ROLE], i+1, str);
-	    	CheckItem(askmenu[RSRC_ASK_ROLE], i+1, currrole == i);
+	    	SetMenuItemText(askmenu[RSRC_ASK_ROLE], i+1, str);
+	    	CheckMenuItem(askmenu[RSRC_ASK_ROLE], i+1, currrole == i);
 	    }
 
 	    /* Adjust the mode popup menu */
-	    CheckItem(askmenu[RSRC_ASK_MODE], 1, currmode == 0);
-	    CheckItem(askmenu[RSRC_ASK_MODE], 2, currmode == 1);
+	    CheckMenuItem(askmenu[RSRC_ASK_MODE], 1, currmode == 0);
+	    CheckMenuItem(askmenu[RSRC_ASK_MODE], 2, currmode == 1);
 #ifdef WIZARD
-	    CheckItem(askmenu[RSRC_ASK_MODE], 3, currmode == 2);
+	    CheckMenuItem(askmenu[RSRC_ASK_MODE], 3, currmode == 2);
 #endif
 
 	    /* Wait for an action on an item */
@@ -703,13 +723,13 @@ void mac_askname ()
 	    case RSRC_ASK_ALIGN:
 	    case RSRC_ASK_GEND:
 	    case RSRC_ASK_MODE:
-	    	GetDItem(askdialog, item, &type, &handle, &rect);
+	    	GetDialogItem(askdialog, item, &type, &handle, &rect);
 	    	pt = *(Point *)&rect;
 	    	LocalToGlobal(&pt);
 	    	if (!!(i = PopUpMenuSelect(askmenu[item], pt.v, pt.h,
 	    			askselect[item] + 1)))
 	    		askselect[item] = LoWord(i) - 1;
-	    	InvalRect(&rect);
+	    	InvalWindowRect(GetDialogWindow(askdialog), &rect);
 	    	break;
 	    case RSRC_ASK_NAME:
 #if 0
@@ -734,8 +754,8 @@ void mac_askname ()
 	} while ((item != RSRC_ASK_PLAY) && (item != RSRC_ASK_QUIT));
 
 	/* Process the name */
-	GetDItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
-	GetIText(handle, str);
+	GetDialogItem(askdialog, RSRC_ASK_NAME, &type, &handle, &rect);
+	GetDialogItemText(handle, str);
 	if (str[0] > PL_NSIZ-1) str[0] = PL_NSIZ-1;
 	BlockMove(&str[1], plname, str[0]);
 	plname[str[0]] = '\0';
@@ -791,22 +811,29 @@ void mac_askname ()
 
 /*** Menu bar routines ***/
 
+#if !TARGET_API_MAC_CARBON
 static void
 alignAD(Rect *pRct, short vExempt)
 {
+	BitMap qbitmap;
+
+
+	GetQDGlobalsScreenBits(&qbitmap);
 	(*pRct).right -= (*pRct).left;		/* width */
 	(*pRct).bottom -= (*pRct).top;		/* height */
-	(*pRct).left = (qd.screenBits.bounds.right - (*pRct).right) / 2;
-	(*pRct).top = (qd.screenBits.bounds.bottom - (*pRct).bottom - vExempt) / 2;
+	(*pRct).left = (qbitmap.bounds.right - (*pRct).right) / 2;
+	(*pRct).top = (qbitmap.bounds.bottom - (*pRct).bottom - vExempt) / 2;
 	(*pRct).top += vExempt;
 	(*pRct).right += (*pRct).left;
 	(*pRct).bottom += (*pRct).top;
 }
+#endif
+
 
 static void
 mustGetMenuAlerts()
 {
-	short		i, mbarHgt = GetMBarHeight();
+	short		i;
 	Rect		**hRct;
 
 	for (i = alrt_Menu_start; i < alrt_Menu_limit; i++)
@@ -818,7 +845,9 @@ mustGetMenuAlerts()
 			ExitToShell();
 		}
 
-		alignAD(*hRct, mbarHgt);
+#if !TARGET_API_MAC_CARBON
+		alignAD(*hRct, GetMBarHeight());
+#endif
 	}
 }
 
@@ -839,41 +868,41 @@ menuError(short menuErr)
 void
 InitMenuRes()
 {
-	static Boolean was_inited = 0 ;
+	static Boolean was_inited = 0;
 	short			i, j;
 	menuListHandle	mlHnd;
-	MenuHandle		mHnd;
+	MenuHandle		menu;
 
 	if (was_inited)
 		return;
-	was_inited = 1 ;
+	was_inited = 1;
 
 	mustGetMenuAlerts();
 
-	for (i = listMenubar; i <= listSubmenu; i++)
-	{
+	for (i = listMenubar; i <= listSubmenu; i++) {
 		if (! (mlHnd = (menuListHandle) GetResource('MNU#', (menuBarListID + i))))
 			menuError(errGetMenuList);
 
-		pMenuList[i] = *mlHnd;
+		pMenuList[i] = (menuListPtr) NewPtr(GetHandleSize((Handle) mlHnd));
+		*pMenuList[i] = **mlHnd;
 
-		for (j = 0; j < (**mlHnd).numMenus; j++)
+		for (j = 0; j < pMenuList[i]->numMenus; j++)
 		{
-			if (! (mHnd = (MenuHandle) GetMenu((**mlHnd).mref[j].mresID))) {
-			Str31 d ;
-				NumToString ( (**mlHnd).mref[j].mresID , d ) ;
+			if (! (menu = (MenuHandle) GetMenu((**mlHnd).mref[j].mresID))) {
+			Str31 d;
+				NumToString ((**mlHnd).mref[j].mresID, d);
 				menuError(errGetMenu);
 			}
 
-			(**mlHnd).mref[j].mhnd = mHnd;
-			* ((short *) *mHnd) = j + (**mlHnd).firstMenuID;	/* consecutive IDs */
+			pMenuList[i]->mref[j].mhnd = menu;
+			SetMenuID(menu, j + (**mlHnd).firstMenuID);	/* consecutive IDs */
 
 			/* expand apple menu */
 			if ((i == listMenubar) && (j == menuApple)) {
-				AppendResMenu(mHnd, 'DRVR');
+				AppendResMenu(menu, 'DRVR');
 			}
 
-			InsertMenu(mHnd, ((i == listSubmenu) ? hierMenu : 0));
+			InsertMenu(menu, ((i == listSubmenu) ? hierMenu : 0));
 		}
 	}
 	DrawMenuBar();
@@ -884,29 +913,23 @@ void
 AdjustMenus(short dimMenubar)
 {
 	short		newMenubar = mbarRegular;
-	WindowPeek	peekWindow = (WindowPeek) FrontWindow();
+	WindowRef win = FrontWindow();
 	short		i;
 
 	/*
-	 *	if ( windowprocs != mac_procs ) {
-	 *		return ;
+	 *	if (windowprocs != mac_procs) {
+	 *		return;
 	 *	}
 	 */
 	/* determine the new menubar state */
-	if (dimMenubar) {
+	if (dimMenubar)
 		newMenubar = mbarDim;
-	} else if (! peekWindow) {
+	else if (!win)
 		newMenubar = mbarNoWindows;
-	} else if (peekWindow->windowKind < 0) {
+	else if (GetWindowKind(win) < 0)
 		newMenubar = mbarDA;
-	} else {
-		while (peekWindow && (peekWindow->windowKind != WKND_MAP)) {
-			peekWindow = peekWindow->nextWindow;
-		}
-		if ((! peekWindow) || (! peekWindow->visible)) {
-			newMenubar = mbarNoMap;
-		}
-	}
+	else if (!IsWindowVisible(_mt_window))
+		newMenubar = mbarNoMap;
 
 	if (newMenubar != mbarRegular)
 		;							/* we've already found its state */
@@ -934,7 +957,7 @@ AdjustMenus(short dimMenubar)
 
 			SetMenuItemText(MHND_FILE, menuFilePlayMode, "\pExplore");
 
-			for (i = CountMItems(MHND_WIZ); i > menuWizardAttributes; i--)
+			for (i = CountMenuItems(MHND_WIZ); i > menuWizardAttributes; i--)
 				DeleteMenuItem(MHND_WIZ, i);
 		}
 	}
@@ -947,33 +970,25 @@ AdjustMenus(short dimMenubar)
 		case mbarDim:
 			/* disable all menus (except the apple menu) */
 			for (i = menuFile; i < NUM_MBAR; i++)
-				DisableItem(MBARHND(i), 0);
+				DisableMenuItem(MBARHND(i), 0);
 			break;
 
 		case mbarNoWindows:
 		case mbarDA:
 		case mbarNoMap:
 			/* enable the file menu, but ... */
-			EnableItem(MHND_FILE, 0);
-
-			if (theMenubar == mbarDA)
-				DisableItem(MHND_FILE, menuFileOpenMap);
-			else
-				EnableItem(MHND_FILE, menuFileOpenMap);
+			EnableMenuItem(MHND_FILE, 0);
 
 			/* ... disable the window commands! */
 			for (i = menuFileRedraw; i <= menuFileEnterExplore; i++)
-				DisableItem(MHND_FILE, i);
-
-			if (theMenubar != mbarNoWindows)
-				EnableItem(MHND_FILE, menuFileClose);
+				DisableMenuItem(MHND_FILE, i);
 
 			/* ... and disable the rest of the menus */
 			for (i = menuEdit; i < NUM_MBAR; i++)
-				DisableItem(MBARHND(i), 0);
+				DisableMenuItem(MBARHND(i), 0);
 
 			if (theMenubar == mbarDA)
-				EnableItem(MHND_EDIT, 0);
+				EnableMenuItem(MHND_EDIT, 0);
 
 			break;
 
@@ -981,22 +996,19 @@ AdjustMenus(short dimMenubar)
 		case mbarSpecial:
 			/* enable all menus ... */
 			for (i = menuFile; i < NUM_MBAR; i++)
-				EnableItem(MBARHND(i), 0);
+				EnableMenuItem(MBARHND(i), 0);
 
 			/* ... except the unused Edit menu */
-			DisableItem(MHND_EDIT, 0);
-
-			/* ... the map is already open! */
-			DisableItem(MHND_FILE, menuFileOpenMap);
+			DisableMenuItem(MHND_EDIT, 0);
 
 			/* ... enable the window commands */
 			for (i = menuFileRedraw; i <= menuFileEnterExplore; i++)
-				EnableItem(MHND_FILE, i);
+				EnableMenuItem(MHND_FILE, i);
 
 			if (theMenubar == mbarRegular)
-				DisableItem(MHND_FILE, menuFilePlayMode);
+				DisableMenuItem(MHND_FILE, menuFilePlayMode);
 			else
-				DisableItem(MHND_FILE, menuFileEnterExplore);
+				DisableMenuItem(MHND_FILE, menuFileEnterExplore);
 
 			break;
 		}
@@ -1016,6 +1028,7 @@ DoMenuEvt(long menuEntry)
 	case menuApple:
 		if (menuItem == menuAppleAboutBox)
 			aboutNetHack();
+#if !TARGET_API_MAC_CARBON
 		else
 		{
 			unsigned char daName[32];
@@ -1023,6 +1036,7 @@ DoMenuEvt(long menuEntry)
 			GetMenuItemText(MHND_APPLE, menuItem, * (Str255 *) daName);
 			(void) OpenDeskAcc(daName);
 		}
+#endif
 		break;
 
 	/*
@@ -1033,28 +1047,20 @@ DoMenuEvt(long menuEntry)
 	case menuFile:
 		switch(menuItem)
 		{
-		case menuFileOpenMap:
-			openMap();
-			break;
-
 		case menuFileRedraw:
-			AddToKeyQueue ( 'R' & 0x1f , 1 ) ;
+			AddToKeyQueue ('R' & 0x1f, 1);
 			break;
 
 		case menuFilePrevMsg:
-			AddToKeyQueue ( 'P' & 0x1f , 1 ) ;
+			AddToKeyQueue ('P' & 0x1f, 1);
 			break;
 
 		case menuFileCleanup:
 			(void) SanePositions();
 			break;
 
-		case menuFileClose:
-			closeFrontWindow();
-			break;
-
 		case menuFileEnterExplore:
-			AddToKeyQueue ( 'X' , 1 ) ;
+			AddToKeyQueue ('X', 1);
 			break;
 
 		case menuFileSave:
@@ -1068,7 +1074,9 @@ DoMenuEvt(long menuEntry)
 		break;
 
 	case menuEdit:
+#if !TARGET_API_MAC_CARBON
 		(void) SystemEdit(menuItem - 1);
+#endif
 		break;
 
 	default:	/* get associated string and add to key queue */
@@ -1095,7 +1103,7 @@ aboutNetHack() {
 	if (theMenubar >= mbarRegular) {
 		(void) doversion();				/* is this necessary? */
 	} else {
-		unsigned char aboutStr[32] = "\pNetHack 3.3.";
+		unsigned char aboutStr[32] = "\pNetHack 3.4.";
 
 		if (PATCHLEVEL > 10) {
 			aboutStr[++aboutStr[0]] = '0'+PATCHLEVEL/10;
@@ -1109,42 +1117,12 @@ aboutNetHack() {
 	}
 }
 
-static void
-openMap()
-{
-	WindowPeek	peekWindow = *(WindowPeek*) LMGetWindowList();
-
-	while (peekWindow && (peekWindow->windowKind != WKND_MAP))
-		peekWindow = peekWindow->nextWindow;
-
-	if (! peekWindow)
-		return;				/* impossible? */
-
-	ShowWindow((WindowPtr) peekWindow);
-	SelectWindow((WindowPtr) peekWindow);
-}
-
-static void
-closeFrontWindow()
-{
-	WindowPeek	peekWindow = (WindowPeek) FrontWindow();
-
-	if (! peekWindow)
-		return;				/* impossible? */
-	else if (peekWindow->windowKind < 0)
-		CloseDeskAcc(peekWindow->windowKind);
-	else if (peekWindow->windowKind == WKND_MAP)
-		HideWindow((WindowPtr) peekWindow);
-	else
-		WindowGoAway((EventRecord *) 0L, (WindowPtr) peekWindow);
-}
-
 
 static void
 askSave()
 {
-	Boolean doSave = 1 ;
-	Boolean doYes = 0 ;
+	Boolean doSave = 1;
+	Boolean doYes = 0;
 
 	if (theMenubar < mbarRegular) {
 	short	itemHit;
@@ -1154,15 +1132,15 @@ askSave()
 		ResetAlertStage();
 
 		if (itemHit != bttnMenuAlertYes) {
-			doSave = 0 ;
+			doSave = 0;
 		} else {
-			doYes = 1 ;
+			doYes = 1;
 		}
 	}
-	if ( doSave ) {
-		AddToKeyQueue ( 'S' , 1 ) ;
-		if ( doYes ) {
-			AddToKeyQueue ( 'y' , 1 ) ;
+	if (doSave) {
+		AddToKeyQueue ('S', 1);
+		if (doYes) {
+			AddToKeyQueue ('y', 1);
 		}
 	}
 }
@@ -1170,29 +1148,48 @@ askSave()
 static void
 askQuit()
 {
-	Boolean doQuit = 1 ;
-	Boolean doYes = 0 ;
-	char *quitinput = "#quit";
+	Boolean doQuit = 1;
+	Boolean doYes = 0;
+	Boolean winMac;
+	char *quitinput;
 
+	if (!strcmp (windowprocs.name, "mac"))
+		winMac = 1;
+	else
+		winMac = 0;
+		
 	if (theMenubar < mbarRegular) {
 	short	itemHit;
 
 		ParamText("\pReally Quit?", "\p", "\p", "\p");
 		itemHit = Alert(alrtMenu_NY, (ModalFilterUPP) 0L);
-		ResetAlrtStage();
+		ResetAlertStage();
 
 		if (itemHit != bttnMenuAlertYes) {
-			doQuit = 0 ;
+			doQuit = 0;
 		} else {
-			doYes = 1 ;
+			doYes = 1;
 		}
 	}
-	if ( doQuit ) {
+	if (doQuit) {
+		/* MWM -- forgive me lord, an even uglier kludge to deal with differences
+			in command input handling
+		 */
+		 if (winMac)
+			quitinput = "#quit\r";
+		else
+			quitinput = "#q\r";
+			
 		/* KMH -- Ugly kludge */
 		while (*quitinput)
 			AddToKeyQueue(*quitinput++, 1);
-		if ( doYes ) {
-			AddToKeyQueue ( 'y' , 1 ) ;
+		if (doYes) {
+			if (winMac)
+				quitinput = "y\rq\r\r\r";
+			else
+				quitinput = "yq\r";
+			while (*quitinput)
+				AddToKeyQueue(*quitinput++, 1);
 		}
 	}
 }
