@@ -8,7 +8,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define yyerrok (yyerrflag=0)
 #define YYRECOVERING (yyerrflag!=0)
 #define YYPREFIX "yy"
-/*	SCCS Id: @(#)dgn_comp.c	3.2	96/06/22	*/
+/*	SCCS Id: @(#)dgn_comp.c	3.4	1996/06/22	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet */
 /*	Copyright (c) 1990 by M. Stephenson				  */
 /* NetHack may be freely redistributed.  See license for details. */
@@ -444,7 +444,7 @@ check_dungeon()
  *	- A level must have a unique level name.
  *	- If chained, the level used as reference for the chain
  *	  must be in this dungeon, must be previously defined, and
- *	  the level chained from must be "non-probabalistic" (ie.
+ *	  the level chained from must be "non-probabilistic" (ie.
  *	  have a 100% chance of existing).
  */
 
@@ -468,8 +468,9 @@ check_level()
 		yyerror("Invaild level chain reference.");
 		return(0);
 	} else if(tmplevel[i].chain != -1) {	/* there is a chain */
-	    if(tmplevel[tmpbranch[i].chain].chance != 100) {
-		yyerror("Level cannot chain from a probabalistic level.");
+	    /* KMH -- tmplevel[tmpbranch[i].chain].chance was in error */
+	    if(tmplevel[tmplevel[i].chain].chance != 100) {
+		yyerror("Level cannot chain from a probabilistic level.");
 		return(0);
 	    } else if(tmplevel[i].chain == n_levs) {
 		yyerror("A level cannot chain to itself!");
@@ -485,7 +486,7 @@ check_level()
  *	  (ie. You can only have one entry point to each dungeon).
  *	- If chained, the level used as reference for the chain
  *	  must be in this dungeon, must be previously defined, and
- *	  the level chained from must be "non-probabalistic" (ie.
+ *	  the level chained from must be "non-probabilistic" (ie.
  *	  have a 100% chance of existing).
  */
 
@@ -513,7 +514,7 @@ check_branch()
 	} else if(tmpbranch[i].chain != -1) {	/* it is chained */
 
 	    if(tmplevel[tmpbranch[i].chain].chance != 100) {
-		yyerror("Branch cannot chain from a probabalistic level.");
+		yyerror("Branch cannot chain from a probabilistic level.");
 		return(0);
 	    }
 	}
