@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)ioctl.c	3.3	90/22/02 */
+/*	SCCS Id: @(#)ioctl.c	3.4	1990/22/02 */
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -72,6 +72,10 @@ struct termio termio;
 extern void NDECL(sco_mapon);
 extern void NDECL(sco_mapoff);
 #endif
+#ifdef __linux__
+extern void NDECL(linux_mapon);
+extern void NDECL(linux_mapoff);
+#endif
 
 #ifdef AUX
 void
@@ -155,6 +159,9 @@ dosuspend()
 		suspend_nhwindows((char *)0);
 #  ifdef _M_UNIX
 		sco_mapon();
+#  endif
+#  ifdef __linux__
+		linux_mapon();
 #  endif
 		(void) signal(SIGTSTP, SIG_DFL);
 #  ifdef AUX
@@ -164,6 +171,9 @@ dosuspend()
 #  endif
 #  ifdef _M_UNIX
 		sco_mapoff();
+#  endif
+#  ifdef __linux__
+		linux_mapoff();
 #  endif
 		resume_nhwindows();
 	} else {
