@@ -5,6 +5,7 @@
 /* This file collects some Unix dependencies; pager.c contains some more */
 
 #include "hack.h"
+#include "wintty.h"
 
 #include	<sys/stat.h>
 #if defined(WIN32) || defined(MSDOS)
@@ -48,14 +49,14 @@ int fd;
 	return(0);
     }
 # else
-#  if defined(MICRO) && !defined(NO_FSTAT)
+#  if (defined(MICRO) || defined(WIN32)) && !defined(NO_FSTAT)
     if(fstat(fd, &buf)) {
 	if(moves > 1) pline("Cannot get status of saved level? ");
-	else pline("Cannot get status of saved game");
+	else pline("Cannot get status of saved game.");
 	return(0);
     } 
     if(comp_times(buf.st_mtime)) { 
-	if(moves > 1) pline("Saved level is out of date");
+	if(moves > 1) pline("Saved level is out of date.");
 	else pline("Saved game is out of date. ");
 	/* This problem occurs enough times we need to give the player
 	 * some more information about what causes it, and how to fix.
@@ -88,6 +89,9 @@ eraseoldlocks()
 		(void) unlink(fqname(lock, LEVELPREFIX, 0));
 	}
 	set_levelfile_name(lock, 0);
+#ifdef HOLD_LOCKFILE_OPEN
+	really_close();
+#endif
 	if(unlink(fqname(lock, LEVELPREFIX, 0)))
 		return 0;				/* cannot remove it */
 	return(1);					/* success! */
@@ -96,28 +100,51 @@ eraseoldlocks()
 void
 getlock()
 {
-	register int fd, c, ci, ct;
+	register int fd, c, ci, ct, ern;
 	char tbuf[BUFSZ];
 	const char *fq_lock;
 # if defined(MSDOS) && defined(NO_TERMS)
 	int grmode = iflags.grmode;
 # endif
-	
 	/* we ignore QUIT and INT at this point */
 	if (!lock_file(HLOCK, LOCKPREFIX, 10)) {
 		wait_synch();
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 		chdirx(orgdir, 0);
+# endif
 		error("Quitting.");
 	}
 
 	/* regularize(lock); */ /* already done in pcmain */
-	Sprintf(tbuf,fqname(lock, LEVELPREFIX, 0));
+	Sprintf(tbuf,"%s",fqname(lock, LEVELPREFIX, 0));
 	set_levelfile_name(lock, 0);
 	fq_lock = fqname(lock, LEVELPREFIX, 1);
 	if((fd = open(fq_lock,0)) == -1) {
 		if(errno == ENOENT) goto gotlock;    /* no such file */
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 		chdirx(orgdir, 0);
+# endif
+# if defined(WIN32) || defined(HOLD_LOCKFILE_OPEN)
+#  if defined(HOLD_LOCKFILE_OPEN)
+ 		if(errno == EACCES) {
+#define OOPS_BUFSZ 512
+ 		    char oops[OOPS_BUFSZ];
+ 		    Strcpy(oops,
+			     "\nThere are files from a game in progress under your name.");
+		    Strcat(oops, "\nThe files are locked or inaccessible.");
+		    Strcat(oops, " Is the other game still running?\n");
+		    if (strlen(fq_lock) < ((OOPS_BUFSZ -16) - strlen(oops)))
+			    Sprintf(eos(oops), "Cannot open %s", fq_lock);
+		    Strcat(oops, "\n");
+		    unlock_file(HLOCK);
+		    error(oops);
+ 		} else
+#  endif
+		error("Bad directory or name: %s\n%s\n",
+				fq_lock, strerror(errno));
+# else
 		perror(fq_lock);
+# endif
 		unlock_file(HLOCK); 
 		error("Cannot open %s", fq_lock);
 	}
@@ -125,9 +152,13 @@ getlock()
 	(void) close(fd);
 
 	if(iflags.window_inited) { 
+# ifdef SELF_RECOVER
+	  c = yn("There are files from a game in progress under your name. Recover?");
+# else
 	  pline("There is already a game in progress under your name.");
 	  pline("You may be able to use \"recover %s\" to get it back.\n",tbuf);
 	  c = yn("Do you want to destroy the old game?");
+# endif
 	} else {
 # if defined(MSDOS) && defined(NO_TERMS)
 		grmode = iflags.grmode;
@@ -135,17 +166,18 @@ getlock()
 # endif
 		c = 'n';
 		ct = 0;
+# ifdef SELF_RECOVER
+		msmsg(
+		"There are files from a game in progress under your name. Recover? [yn]");
+# else
 		msmsg("\nThere is already a game in progress under your name.\n");
 		msmsg("If this is unexpected, you may be able to use \n");
 		msmsg("\"recover %s\" to get it back.",tbuf);
 		msmsg("\nDo you want to destroy the old game? [yn] ");
+# endif
 		while ((ci=nhgetch()) != '\n') {
 		    if (ct > 0) {
-# if defined(WIN32CON)
-			backsp();       /* \b is visible on NT */
-# else
 			msmsg("\b \b");
-# endif
 			ct = 0;
 			c = 'n';
 		    }
@@ -157,36 +189,68 @@ getlock()
 		}
 	}
 	if(c == 'y' || c == 'Y')
+# ifndef SELF_RECOVER
 		if(eraseoldlocks()) {
-# if defined(WIN32CON)
+#  if defined(WIN32CON)
 			clear_screen();		/* display gets fouled up otherwise */
-# endif
+#  endif
 			goto gotlock;
 		} else {
 			unlock_file(HLOCK);
+#  if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 			chdirx(orgdir, 0);
+#  endif
 			error("Couldn't destroy old game.");
 		}
+# else /*SELF_RECOVER*/
+		if(recover_savefile()) {
+#  if defined(WIN32CON)
+			clear_screen();		/* display gets fouled up otherwise */
+#  endif
+			goto gotlock;
+		} else {
+			unlock_file(HLOCK);
+#  if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
+			chdirx(orgdir, 0);
+#  endif
+			error("Couldn't recover old game.");
+		}
+# endif /*SELF_RECOVER*/
 	else {
 		unlock_file(HLOCK);
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 		chdirx(orgdir, 0);
-		error("%s", "");
+# endif
+		error("%s", "Cannot start a new game.");
 	}
 
 gotlock:
 	fd = creat(fq_lock, FCMASK);
+	if (fd == -1) ern = errno;
 	unlock_file(HLOCK);
 	if(fd == -1) {
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 		chdirx(orgdir, 0);
-		error("cannot creat lock file (%s.)", fq_lock);
+# endif
+# if defined(WIN32)
+		error("cannot creat file (%s.)\n%s\n%s\"%s\" exists?\n", 
+				fq_lock, strerror(ern), " Are you sure that the directory",
+				fqn_prefix[LEVELPREFIX]);
+# else
+		error("cannot creat file (%s.)", fq_lock);
+# endif
 	} else {
 		if(write(fd, (char *) &hackpid, sizeof(hackpid))
 		    != sizeof(hackpid)){
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 			chdirx(orgdir, 0);
+# endif
 			error("cannot write lock (%s)", fq_lock);
 		}
 		if(close(fd) == -1) {
+# if defined(CHDIR) && !defined(NOCWD_ASSUMPTIONS)
 			chdirx(orgdir, 0);
+# endif
 			error("cannot close lock (%s)", fq_lock);
 		}
 	}
@@ -194,7 +258,7 @@ gotlock:
 	if (grmode) gr_init();
 # endif
 }	
-# endif /* PC_LOCKING */
+#endif /* PC_LOCKING */
 
 # ifndef WIN32
 void
