@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)winnt.c	 3.4	 1997/04/12		  */
+/*	SCCS Id: @(#)winnt.c	 3.4	 $Date: 2003/10/26 15:58:22 $		  */
 /* Copyright (c) NetHack PC Development Team 1993, 1994 */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -17,7 +17,9 @@
 #endif
 #include <ctype.h>
 #include "win32api.h"
-
+#ifdef WIN32CON
+#include "wintty.h"
+#endif
 #ifdef WIN32
 
 
@@ -38,7 +40,6 @@
 HANDLE ffhandle = (HANDLE)0;
 WIN32_FIND_DATA ffd;
 
-
 /* The function pointer nt_kbhit contains a kbhit() equivalent
  * which varies depending on which window port is active.
  * For the tty port it is tty_kbhit() [from nttty.c]
@@ -151,10 +152,7 @@ def_kbhit()
 }
 
 /* 
- * Windows NT version >= 3.5x and above supports long file names,
- * even on FAT volumes (VFAT), so no need for nt_regularize.
- * Windows NT 3.1 could not do long file names except on NTFS,
- * so nt_regularize was required.
+ * Strip out troublesome file system characters.
  */
 
 void
@@ -199,27 +197,128 @@ return &szFullPath[0];
 }
 # endif
 
-
+#ifndef WIN32CON
 /* fatal error */
 /*VARARGS1*/
-
 void
 error VA_DECL(const char *,s)
+	char buf[BUFSZ];
 	VA_START(s);
 	VA_INIT(s, const char *);
 	/* error() may get called before tty is initialized */
 	if (iflags.window_inited) end_screen();
-	putchar('\n');
-	Vprintf(s,VA_ARGS);
-	putchar('\n');
+	if (!strncmpi(windowprocs.name, "tty", 3)) {
+		buf[0] = '\n';
+		(void) vsprintf(&buf[1], s, VA_ARGS);
+		Strcat(buf, "\n");
+		msmsg(buf);
+	} else {
+		(void) vsprintf(buf, s, VA_ARGS);
+		Strcat(buf, "\n");
+		raw_printf(buf);
+	}
 	VA_END();
 	exit(EXIT_FAILURE);
 }
+#endif
 
 void Delay(int ms)
 {
 	(void)Sleep(ms);
 }
+
+#ifdef WIN32CON
+extern void NDECL(backsp);
+#endif
+
+void win32_abort()
+{
+#ifdef WIZARD
+   	if (wizard) {
+# ifdef WIN32CON
+	    int c, ci, ct;
+
+   	    if (!iflags.window_inited)
+		c = 'n';
+		ct = 0;
+		msmsg("Execute debug breakpoint wizard?");
+		while ((ci=nhgetch()) != '\n') {
+		    if (ct > 0) {
+			backsp();       /* \b is visible on NT */
+			(void) putchar(' ');
+			backsp();
+			ct = 0;
+			c = 'n';
+		    }
+		    if (ci == 'y' || ci == 'n' || ci == 'Y' || ci == 'N') {
+		    	ct = 1;
+		        c = ci;
+		        msmsg("%c",c);
+		    }
+		}
+		if (c == 'y')
+			DebugBreak();
+# endif
+	}
+#endif
+	abort();
+}
+
+static char interjection_buf[INTERJECTION_TYPES][1024];
+static int interjection[INTERJECTION_TYPES];
+
+void
+interject_assistance(num, interjection_type, ptr1, ptr2)
+int num;
+int interjection_type;
+genericptr_t ptr1;
+genericptr_t ptr2;
+{
+	switch(num) {
+	    case 1: {
+		char *panicmsg = (char *)ptr1;
+		char *datadir =  (char *)ptr2;
+		char *tempdir = nh_getenv("TEMP");
+		interjection_type = INTERJECT_PANIC;
+		interjection[INTERJECT_PANIC] = 1;
+		/*
+		 * ptr1 = the panic message about to be delivered.
+		 * ptr2 = the directory prefix of the dungeon file
+		 *        that failed to open.
+		 * Check to see if datadir matches tempdir or a
+		 * common windows temp location. If it does, inform
+		 * the user that they are probably trying to run the
+		 * game from within their unzip utility, so the required
+		 * files really don't exist at the location. Instruct
+		 * them to unpack them first.
+		 */
+		if (panicmsg && datadir) {
+		    if (!strncmpi(datadir, "C:\\WINDOWS\\TEMP", 15) ||
+			    strstri(datadir, "TEMP")   ||
+			    (tempdir && strstri(datadir, tempdir))) {
+			(void)strncpy(interjection_buf[INTERJECT_PANIC],
+			"\nOne common cause of this error is attempting to execute\n"
+			"the game by double-clicking on it while it is displayed\n"
+			"inside an unzip utility.\n\n"
+			"You have to unzip the contents of the zip file into a\n"
+			"folder on your system, and then run \"NetHack.exe\" or \n"
+			"\"NetHackW.exe\" from there.\n\n"
+			"If that is not the situation, you are encouraged to\n"
+			"report the error as shown above.\n\n", 1023);
+		    }
+		}
+	    }
+	    break;
+	}
+}
+
+void
+interject(interjection_type)
+int interjection_type;
+{
+	if (interjection_type >= 0 && interjection_type < INTERJECTION_TYPES)
+		msmsg(interjection_buf[interjection_type]);
+}
 #endif /* WIN32 */
 
 /*winnt.c*/
