@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)makedefs.c	3.4	2002/03/03	*/
+/*	SCCS Id: @(#)makedefs.c	3.4	2002/08/14	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* Copyright (c) M. Stephenson, 1990, 1991.			  */
 /* Copyright (c) Dean Luick, 1990.				  */
@@ -78,7 +78,8 @@ static	const char	SCCS_Id[] = "@(#)makedefs.c\t3.4\t2002/02/03";
 # define DATA_TEMPLATE		"NH:slib/%s"
 # define DATA_IN_TEMPLATE	"NH:dat/%s"
 #else /* not AMIGA */
-# ifdef MAC
+# if defined(MAC) && !defined(__MACH__)
+    /* MacOS 9 or earlier */
 #   define INCLUDE_TEMPLATE	":include:%s"
 #   define SOURCE_TEMPLATE	":src:%s"
 #   define DGN_TEMPLATE		":dat:%s"  /* where dungeon.pdf file goes */
@@ -170,6 +171,7 @@ static boolean FDECL(d_filter, (char *));
 static boolean FDECL(h_filter, (char *));
 static boolean FDECL(ranged_attk,(struct permonst*));
 static int FDECL(mstrength,(struct permonst *));
+static void NDECL(build_savebones_compat_string);
 
 static boolean FDECL(qt_comment, (char *));
 static boolean FDECL(qt_control, (char *));
@@ -406,6 +408,18 @@ do_rumors()
 	return;
 }
 
+/*
+ * 3.4.1: way back in 3.2.1 `flags.nap' became unconditional but
+ * TIMED_DELAY was erroneously left in VERSION_FEATURES and has
+ * been there up through 3.4.0.  Simply removing it now would
+ * break save file compatibility with 3.4.0 files, so we will
+ * explicitly mask it out during version checks.
+ * This should go away in the next version update.
+ */
+#define IGNORED_FEATURES	( 0L \
+				| (1L << 23)	/* TIMED_DELAY */ \
+				)
+
 static void
 make_version()
 {
@@ -465,9 +479,6 @@ make_version()
 #endif
 #ifdef SCORE_ON_BOTL
 			| (1L << 21)
-#endif
-#ifdef TIMED_DELAY
-			| (1L << 23)
 #endif
 		/* data format [COMPRESS excluded] (27..31) */
 #ifdef ZEROCOMP
@@ -569,6 +580,10 @@ do_date()
 		version.incarnation, ul_sfx);
 	Fprintf(ofp,"#define VERSION_FEATURES 0x%08lx%s\n",
 		version.feature_set, ul_sfx);
+#ifdef IGNORED_FEATURES
+	Fprintf(ofp,"#define IGNORED_FEATURES 0x%08lx%s\n",
+		(unsigned long) IGNORED_FEATURES, ul_sfx);
+#endif
 	Fprintf(ofp,"#define VERSION_SANITY1 0x%08lx%s\n",
 		version.entity_count, ul_sfx);
 	Fprintf(ofp,"#define VERSION_SANITY2 0x%08lx%s\n",
@@ -591,6 +606,27 @@ do_date()
 	return;
 }
 
+static char save_bones_compat_buf[BUFSZ];
+
+static void
+build_savebones_compat_string()
+{
+#ifdef VERSION_COMPATIBILITY
+	unsigned long uver = VERSION_COMPATIBILITY;
+#endif
+	Strcpy(save_bones_compat_buf,
+		"save and bones files accepted from version");
+#ifdef VERSION_COMPATIBILITY
+	Sprintf(eos(save_bones_compat_buf), "s %lu.%lu.%lu through %d.%d.%d",
+		((uver & 0xFF000000L) >> 24), ((uver & 0x00FF0000L) >> 16),
+		((uver & 0x0000FF00L) >> 8),
+		VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL);
+#else
+	Sprintf(eos(save_bones_compat_buf), " %d.%d.%d only",
+		VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL);
+#endif
+}
+
 static const char *build_opts[] = {
 #ifdef AMIGA_WBENCH
 		"Amiga WorkBench support",
@@ -598,6 +634,9 @@ static const char *build_opts[] = {
 #ifdef ANSI_DEFAULT
 		"ANSI default terminal",
 #endif
+#ifdef AUTOPICKUP_EXCEPTIONS
+		"autopickup_exceptions",
+#endif
 #ifdef TEXTCOLOR
 		"color",
 #endif
@@ -631,6 +670,9 @@ static const char *build_opts[] = {
 #ifdef KOPS
 		"Keystone Kops",
 #endif
+#ifdef HOLD_LOCKFILE_OPEN
+		"exclusive lock on level 0 file",
+#endif
 #ifdef LOGFILE
 		"log file",
 #endif
@@ -703,7 +745,7 @@ static const char *build_opts[] = {
 #ifdef TERMINFO
 		"terminal info library",
 #else
-# if defined(TERMLIB) || (!defined(MICRO) && defined(TTY_GRAPHICS))
+# if defined(TERMLIB) || ((!defined(MICRO) && !defined(WIN32)) && defined(TTY_GRAPHICS))
 		"terminal capability library",
 # endif
 #endif
@@ -713,6 +755,13 @@ static const char *build_opts[] = {
 #ifdef TOURIST
 		"tourists",
 #endif
+#ifdef USER_SOUNDS
+# ifdef USER_SOUNDS_REGEX
+		"user sounds via regular expressions",
+# else
+		"user sounds via pmatch",
+# endif
+#endif
 #ifdef PREFIXES_IN_USE
 		"variable playground",
 #endif
@@ -725,6 +774,7 @@ static const char *build_opts[] = {
 #ifdef ZEROCOMP
 		"zero-compressed save files",
 #endif
+		save_bones_compat_buf,
 		"basic NetHack features"
 	};
 
@@ -775,6 +825,7 @@ do_options()
 		exit(EXIT_FAILURE);
 	}
 
+	build_savebones_compat_string();
 	Fprintf(ofp,
 #ifdef BETA
 		"\n    NetHack version %d.%d.%d [beta]\n",
@@ -1495,6 +1546,9 @@ static void
 do_qt_text(s)
 	char *s;
 {
+	if (!in_msg) {
+	    Fprintf(stderr, TEXT_NOT_IN_MSG, qt_line);
+	}
 	curr_msg->size += strlen(s);
 	return;
 }
