@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)tiletext.c	3.3	1999/10/24	*/
+/*	SCCS Id: @(#)tiletext.c	3.4	1999/10/24	*/
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "config.h"
@@ -18,9 +18,9 @@ static pixel placeholder[TILE_Y][TILE_X];
 static FILE *tile_file;
 static int tile_set, tile_set_indx;
 #if (TILE_X==8)
-static char *text_sets[] = { "monthin.txt", "objthin.txt", "oththin.txt" };
+static const char *text_sets[] = { "monthin.txt", "objthin.txt", "oththin.txt" };
 #else
-static char *text_sets[] = { "monsters.txt", "objects.txt", "other.txt" };
+static const char *text_sets[] = { "monsters.txt", "objects.txt", "other.txt" };
 #endif
 
 extern const char *FDECL(tilename, (int, int));
