@@ -835,10 +835,6 @@ cl_eos()			/* free after Robert Viduya */
 
 #include <curses.h>
 
-#ifndef LINUX
-extern char *tparm();
-#endif
-
 #  ifdef COLOR_BLACK	/* trust include file */
 #undef COLOR_BLACK
 #  else
@@ -1072,6 +1068,9 @@ int n;
 		    if(nh_US) return nh_US;
 	    case ATR_BOLD:
 	    case ATR_BLINK:
+#if defined(TERMLIB) && defined(TEXTCOLOR)
+		    if (MD) return MD;
+#endif
 		    return nh_HI;
 	    case ATR_INVERSE:
 		    return MR;
@@ -1161,6 +1160,11 @@ int color;
 	if (windowprocs.name != NULL &&
 	    !strcmpi(windowprocs.name, "Gem")) return TRUE;
 #endif
+#ifdef QT_GRAPHICS
+	/* XXX has_color() should be added to windowprocs */
+	if (windowprocs.name != NULL &&
+	    !strcmpi(windowprocs.name, "Qt")) return TRUE;
+#endif
 #ifdef AMII_GRAPHICS
 	/* hilites[] not used */
 	return iflags.use_color;
