@@ -18,6 +18,7 @@ typedef struct mswin_nethack_main_window {
 
 static TCHAR szMainWindowClass[] = TEXT("MSNHMainWndClass");
 static TCHAR szTitle[MAX_LOADSTRING];
+extern void mswin_display_splash_window(BOOL);
 
 LRESULT CALLBACK	MainWndProc(HWND, UINT, WPARAM, LPARAM);
 LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
@@ -30,6 +31,7 @@ static int		mapmode2menuid(int map_mode);
 HWND mswin_init_main_window () {
 	static int run_once = 0;
 	HWND ret;
+    WINDOWPLACEMENT wp;
 
 	/* register window class */
 	if( !run_once ) {
@@ -55,6 +57,28 @@ HWND mswin_init_main_window () {
 
 	if( !ret ) panic("Cannot create main window");
 
+    
+    if (GetNHApp()->regMainMinX != CW_USEDEFAULT)
+    {
+        wp.length = sizeof(wp);
+        wp.showCmd = GetNHApp()->regMainShowState;
+
+        wp.ptMinPosition.x = GetNHApp()->regMainMinX;
+        wp.ptMinPosition.y = GetNHApp()->regMainMinY;
+
+        wp.ptMaxPosition.x = GetNHApp()->regMainMaxX;
+        wp.ptMaxPosition.y = GetNHApp()->regMainMaxY;
+
+        wp.rcNormalPosition.left = GetNHApp()->regMainLeft;
+        wp.rcNormalPosition.top = GetNHApp()->regMainTop;
+        wp.rcNormalPosition.right = GetNHApp()->regMainRight;
+        wp.rcNormalPosition.bottom = GetNHApp()->regMainBottom;
+        SetWindowPlacement(ret, &wp);
+    }
+    else
+        ShowWindow(ret, SW_SHOWDEFAULT);
+    UpdateWindow(ret);
+
 	return ret;
 }
 
@@ -113,13 +137,13 @@ numpad[KEY_LAST][3] = {
 	{'9', M('9'), '9'}, /* 9 */
 	{'m', C('p'), C('p')}, /* - */
 	{'4', M('4'), '4'}, /* 4 */
-	{'g', 'G', 'g'}, /* 5 */
+	{'5', M('5'), '5'}, /* 5 */
 	{'6', M('6'), '6'}, /* 6 */
 	{'+', 'P', C('p')}, /* + */
 	{'1', M('1'), '1'}, /* 1 */
 	{'2', M('2'), '2'}, /* 2 */
 	{'3', M('3'), '3'}, /* 3 */
-	{'i', 'I', C('i')}, /* Ins */
+	{'0', M('0'), '0'}, /* Ins */
 	{'.', ':', ':'} /* Del */
 };
 
@@ -406,9 +430,29 @@ LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lPar
 
 		case WM_MOVE:
 		case WM_SIZE:
-			mswin_layout_main_window(NULL);
-			break;
+        {
+            WINDOWPLACEMENT wp;
 
+			mswin_layout_main_window(NULL);
+            
+            wp.length = sizeof(wp);
+            if (GetWindowPlacement(hWnd, &wp)) {
+                GetNHApp()->regMainShowState = (wp.showCmd == SW_SHOWMAXIMIZED 
+		    ? SW_SHOWMAXIMIZED : SW_SHOWNORMAL);
+
+                GetNHApp()->regMainMinX = wp.ptMinPosition.x;
+                GetNHApp()->regMainMinY = wp.ptMinPosition.y;
+
+                GetNHApp()->regMainMaxX = wp.ptMaxPosition.x;
+                GetNHApp()->regMainMaxY = wp.ptMaxPosition.y;
+
+                GetNHApp()->regMainLeft = wp.rcNormalPosition.left;
+                GetNHApp()->regMainTop = wp.rcNormalPosition.top;
+                GetNHApp()->regMainRight = wp.rcNormalPosition.right;
+                GetNHApp()->regMainBottom = wp.rcNormalPosition.bottom;
+            }
+			break;
+        }
 		case WM_SETFOCUS:
 			/* if there is a menu window out there -
 			   transfer input focus to it */
@@ -428,9 +472,15 @@ LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lPar
 			    NHEVENT_KBD('\033'); /* and send keyboard input as if user pressed ESC */
 			    /* additional code for this is done in menu and rip windows */
 			}
+			else if (!program_state.something_worth_saving)
+			{
+			    /* User exited before the game started, e.g. during splash display */
+			    /* Just get out. */
+			    bail((char *)0);
+			}
 			else
 			{
-			    switch(MessageBox(hWnd, TEXT("Save?"), TEXT("NetHack for Windows"), MB_YESNOCANCEL | MB_ICONQUESTION)) {
+			    switch (NHMessageBox(hWnd, TEXT("Save?"), MB_YESNOCANCEL | MB_ICONQUESTION)) {
 			    case IDYES: NHEVENT_KBD('y'); dosave(); break;
 			    case IDNO: NHEVENT_KBD('q'); done(QUIT); break;
 			    case IDCANCEL: break;
@@ -608,6 +658,7 @@ void mswin_layout_main_window(HWND changed_child)
 						   TRUE );
 				break;
 
+			case NHW_TEXT: // same as the map window
 			case NHW_MAP:
 				MoveWindow(GetNHApp()->windowlist[i].win, 
 					       map_org.x, 
@@ -632,7 +683,6 @@ void mswin_layout_main_window(HWND changed_child)
 
 				pt.x = map_org.x + max(0, (int)(map_size.cx-menu_size.cx));
 				pt.y = map_org.y;
-				ClientToScreen(GetNHApp()->hMainWnd, &pt);
 				MoveWindow(GetNHApp()->windowlist[i].win, 
 						   pt.x, 
 						   pt.y,
@@ -659,7 +709,7 @@ LRESULT onWMCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	switch (wmId)
 	{
 		case IDM_ABOUT:
-		   DialogBox(GetNHApp()->hApp, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
+     mswin_display_splash_window(TRUE);
 		   break;
 
 		case IDM_EXIT:
@@ -667,7 +717,8 @@ LRESULT onWMCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		   break;
 
 		case IDM_SAVE:
-		   dosave();
+		   if (!program_state.gameover && !program_state.done_hup) dosave();
+		   else MessageBeep(0);
 		   break;
 
 		case IDM_MAP_TILES:
@@ -709,9 +760,9 @@ LRESULT onWMCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
         {
             mswin_destroy_reg();
             /* Notify the user that windows settings will not be saved this time. */
-            MessageBox(GetNHApp()->hMainWnd, 
+            NHMessageBox(GetNHApp()->hMainWnd, 
                 "Your Windows Settings will not be stored when you exit this time.", 
-                "NetHack", MB_OK | MB_ICONINFORMATION);
+                MB_OK | MB_ICONINFORMATION);
             break;
         }
 		case IDM_HELP_LONG:	
